// SpaceAntShower.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// VINCIA is licenced under the GNU GPL version 2.
// Copyright (C) 2017 P. Skands.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Function definitions (not found in the header) for the SpaceAntShower
// class, as follows:
// 1) inherited member functions from the Pythia 8 SpaceShower classs
// 2) non-inherited member functions for the SpaceAntShower class

// Include the Vincia headers 
#include "Vincia/SpaceAntShower.h"
#include "Vincia/TimeAntShower.h"

namespace Vincia {

//**********************************************************************

// The SpaceAntShower class and its members

//**********************************************************************

// BEGIN INHERITED FUNCTIONS FOR INTERFACE WITH PYTHIA8
//  : init     : global shower initialization function
//  : prepare  : prepare partonic system for showering
//  : update   : update the partonic system
//  : pTnext   : generate next scale in downwards evolution
//  : branch   : perform a branching, store results in event record

//*********

// Initialize Shower

void SpaceAntShower::init( BeamParticle* beamAPtrIn, 
  BeamParticle* beamBPtrIn) {

  // Check if already initialized 
  if (isInit) 
    if (settingsPtr->word("Merging:Process").compare("void") == 0)
      return;

  // Event counters and debugging
  nAccepted       = -1; // Set to -1, so prepare works properly
  nSelected       = -1;
  nVetoUserHooks  = 0;
  nFailHadLevel   = 0;
  nCallPythiaNext = 0;

  // Verbose level and version number
  verbose = settingsPtr->mode("Vincia:verbose");
  version = settingsPtr->parm("Vincia:versionNumber");

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::init","begin --------------");

  // Beam parameters
  beamFrameType = settingsPtr->mode("Beams:frameType");

  // Beam particles
  beamAPtr = beamAPtrIn;
  beamBPtr = beamBPtrIn;
  // Make sure that A is the guy with p+
  bool swap = false;
  if ( beamAPtr->pz() < 0.0 ) swap = true;
  if (swap) { beamAPtr = beamBPtrIn; beamBPtr = beamAPtrIn; }
  sbbSav = m2(beamAPtr->p(), beamBPtr->p());
  eBeamA = beamAPtr->e();
  eBeamB = beamBPtr->e();

  // Possibility to allow user veto of emission step.
  hasUserHooks      = (userHooksPtr != 0);
  canVetoEmission   = (hasUserHooks && userHooksPtr->canVetoISREmission());

  // Number of active quark flavours
  nGluonToQuarkI    = settingsPtr->mode("Vincia:nGluonToQuarkI");
  nGluonToQuarkF    = settingsPtr->mode("Vincia:nGluonToQuarkF");

  // Perturbative cutoff
  cutoffScaleII     = settingsPtr->parm("Vincia:cutoffScaleII");
  cutoffScaleIF     = settingsPtr->parm("Vincia:cutoffScaleIF");

  // Normalization factors
  evNormIIpT2       = settingsPtr->parm("Vincia:pTnormalizationII");
  evNormIFpT2       = settingsPtr->parm("Vincia:pTnormalizationIF");
  evNormMsplitI     = settingsPtr->parm("Vincia:mSnormalizationI");
  evNormMconvI      = settingsPtr->parm("Vincia:mCnormalizationI");
  evNormMsplitF     = settingsPtr->parm("Vincia:mSnormalizationF");
  evAllPT           = settingsPtr->flag("Vincia:evolveAllInPT");

  // Mass corrections
  nFlavZeroMass       = settingsPtr->mode("Vincia:nFlavZeroMassI");
  massCorrectionLevel = settingsPtr->mode("Vincia:massCorrectionLevelI");
  // Safety checks
  if (nFlavZeroMass == 5) massCorrectionLevel = 0;
  if (massCorrectionLevel == 0) nFlavZeroMass = 5;
  if ( (settingsPtr->mode("PDFinProcess:nQuarkIn") <= nFlavZeroMass) && 
    (nGluonToQuarkI <= nFlavZeroMass) && (nGluonToQuarkF <= nFlavZeroMass) )
    massCorrectionLevel = 0;

  // Mass windows : specifies thresholds for each flavour.
  mt          = vinComPtr->mt;
  mb          = vinComPtr->mb;
  mc          = vinComPtr->mc;
  ms          = vinComPtr->ms;
  // Derived scales
  mtb         = sqrt(mt*mb);

  // Set shower alphaS pointer
  useCMW                = settingsPtr->flag("Vincia:useCMW");
  alphaSptr             = &vinComPtr->alphaStrong;
  if (useCMW) alphaSptr = &vinComPtr->alphaStrongCMW;

  // AlphaS parameters
  alphaSorder        = settingsPtr->mode("Vincia:alphaSorder");
  alphaSvalue        = settingsPtr->parm("Vincia:alphaSvalue");
  aSkMu2             = pow2(settingsPtr->parm("Vincia:alphaSkMuI"));
  aSkMu2SplitI       = pow2(settingsPtr->parm("Vincia:alphaSkMuSplitI"));
  aSkMu2Conv         = pow2(settingsPtr->parm("Vincia:alphaSkMuConv"));
  aSkMu2SplitF       = pow2(settingsPtr->parm("Vincia:alphaSkMuSplitF"));
  alphaSmax          = settingsPtr->parm("Vincia:alphaSmax");
  alphaSmuFreeze     = settingsPtr->parm("Vincia:alphaSmuFreeze");
  mu2freeze          = pow2(alphaSmuFreeze);

  // Smallest allow scale
  alphaSmuMin        = max( alphaSmuFreeze, 1.05 *alphaSptr->Lambda3() );
  mu2min             = pow2(alphaSmuMin);

  // Check largest numerical value of alphaS we can have
  if (alphaSorder >= 1)
    alphaSmax = min( alphaSmax, alphaSptr->alphaS(mu2min) );

  // If we want to get rid of heavy quarks we need to change the masses
  // to the ones in the pdfs!
  BeamParticle* beamUsePtr =
    ( (abs(beamAPtr->id()) < 100) ? beamBPtr : beamAPtr );
  if ((abs(beamUsePtr->id()) > 100) && (massCorrectionLevel > 0)
    && (nFlavZeroMass < 5)) {
    vector<double> masses;
    masses.resize(2);
    masses[0] = settingsPtr->parm("Vincia:ThresholdMB");
    masses[1] = settingsPtr->parm("Vincia:ThresholdMC");
    for (int i=0; i<(5-nFlavZeroMass); i++) {
      // Check if we can get mass from beams directly (LHAPDF6 only)
      if (beamUsePtr->mQuarkPDF(5-i) > 0.0) {
        masses[i] = beamUsePtr->mQuarkPDF(5-i);
        continue;
      }
      // If not attempt to find it
      double startMass  = masses[i];
      int maxTry        = 500;
      double scale2Last = pow2(startMass);
      double xfLast     = beamUsePtr->xf(5-i,0.001,scale2Last);
      //double xfLast    = beamUsePtr->xf(5-i,0.0001,pow2(startMass));
      for (int j=1; j<=maxTry; j++) {
        double scale2Now = pow2( startMass + 0.005*((double)(j)) );
        double xfNow     = beamUsePtr->xf(5-i,0.001,scale2Now);
        //double xfNow     = beamUsePtr->xf(5-i,0.0001,scale2Now);
        // Method 1: x = 0.001 and check the gradient
        if ( (xfNow-xfLast) > TINY) {
          masses[i] = sqrt(scale2Last);
          break;
        }
        /*// Method 2: x = 0.0001 and check the gradient
          if ( (xfNow-xfLast) > TINY) {
          masses[i] = sqrt(scale2Last);
          break;
          }
          // Method 3: x = 0.001 and check the absolut value
          if (xfNow  > TINY) {
          masses[i] = sqrt(scale2Last);
          break;
          }
          // Method 4: x = 0.0001 and check the absolut value
          if (xfNow > TINY) {
          masses[i] = sqrt(scale2Last);
          break;
          }*/
        // Update
        scale2Last = scale2Now;
        xfLast     = xfNow;
      }
    }
    for (int i=0; i<(int)masses.size(); i++) {
      (i==0 ? mb : mc) = masses[i];
      if (verbose >= 3)
        printOut("SpaceAntShower::init","Found "+num2str(masses[i])+
          (i==0 ? " as b mass." : " as c mass."));
    }
    // Reset the derived scales
    ms  = min(mc,ms);
    mtb = sqrt(mt*mb);
  }

  // Evolution windows
  regMinScalesMtSav.clear();
  regMinScalesSav.clear();
  regMinScalesNow.clear();
  // Fill constant version with masses
  regMinScalesMtSav.push_back(mc/16.0);
  regMinScalesMtSav.push_back(mc/5.0);
  regMinScalesMtSav.push_back(mc);
  regMinScalesMtSav.push_back(mb);
  regMinScalesMtSav.push_back(mtb);
  regMinScalesMtSav.push_back(mt);
  // Fill the rest
  regMinScalesSav = regMinScalesMtSav;
  double qMinNow = 2.0*mt;
  double multFac = 2.0;
  double sqrtS   = sqrt(sbbSav);
  while (qMinNow < sqrtS) {
    int iRegNew = int( log(qMinNow/mt)/log(5) + 5);
    int iRegMax = (int)regMinScalesSav.size()-1;
    if (iRegNew > iRegMax) 
      regMinScalesSav.push_back( pow(5,(double)(iRegMax+1)-5.0)*mt );
    qMinNow *= multFac;
  }

  // Trial Generators
  vector<TrialGeneratorISR*> trialGenerators;
  trialGenerators.push_back(&trialIISoft);
  trialGenerators.push_back(&trialIIGCollA);
  trialGenerators.push_back(&trialIIGCollB);
  trialGenerators.push_back(&trialIISplitA);
  trialGenerators.push_back(&trialIISplitB);
  trialGenerators.push_back(&trialIIConvA);
  trialGenerators.push_back(&trialIIConvB);
  trialGenerators.push_back(&trialIFSoft);
  trialGenerators.push_back(&trialIFQCollA);
  trialGenerators.push_back(&trialIFQCollK);
  trialGenerators.push_back(&trialIFGCollA1);
  trialGenerators.push_back(&trialIFGCollA2);
  trialGenerators.push_back(&trialIFGCollK);
  trialGenerators.push_back(&trialIFSplitA);
  trialGenerators.push_back(&trialIFSplitK);
  trialGenerators.push_back(&trialIFConvA);
  for (int indx=0; indx < int(trialGenerators.size()); ++indx) {
    trialGenerators[indx]->initPtr(rndmPtr,settingsPtr,infoPtr);
    trialGenerators[indx]->init(mc,mb);
  }

  // Uncertainty Bands and Weights
  uncertaintyBands   = settingsPtr->flag("Vincia:uncertaintyBands");
  allowReweighting   = settingsPtr->flag("Vincia:allowReweighting");

  // Global flag for helicity dependence
  helicityShower     = settingsPtr->flag("Vincia:helicityShower");

  // local flag for polarization and mass. Default values
  isMassless = true;    
  usePolarizationNow = helicityShower;

  // Colour reconnections
  CRmode        = settingsPtr->mode("Vincia:CRmode");
  if (CRmode > 0) { // TODO no CR at the moment
    printOut("SpaceAntShower::init","warning; forcing CRmode == 0 "
      "since disabled at the moment.");
    settingsPtr->mode("Vincia:CRmode",0);
    CRmode = 0;
  }
  CRjunctions   = settingsPtr->flag("Vincia:CRjunctions");

  // Matching settings
  matchingLO         = settingsPtr->mode("Vincia:matchingLO");
  meCorrections      = (matchingLO >= 1);
  meCorrectionsNow   = meCorrections;
  matchingFullColor  = settingsPtr->flag("Vincia:matchingFullColor");
  matchingRegOrder   = settingsPtr->mode("Vincia:matchingRegOrder");
  // Matching scale can only be imposed at highest matched multiplicity
  // or not at all. (Matching assumes all previous multiplicities were
  // matched over all of phase space.)
  if (matchingRegOrder <= matchingLO-1) {
    printOut("SpaceAntShower::init","warning; forcing matchingRegOrder >= "
      "matchingLO");
    matchingRegOrder = max(matchingLO,matchingRegOrder);
  }
  matchingRegShape   = settingsPtr->mode("Vincia:matchingRegShape");
  matchingScaleIsAbs = settingsPtr->flag("Vincia:matchingRegScaleIsAbsolute");
  matchingScale      = settingsPtr->parm("Vincia:matchingRegScale");
  matchingScaleRatio = settingsPtr->parm("Vincia:matchingRegScaleRatio");
  matchingIRcutoff   = settingsPtr->parm("Vincia:matchingIRcutoff");
  Hinterference      = settingsPtr->flag("Vincia:Hinterference");

  // Initialize headroom factors (optimized dynamically later?)
  optimizeHeadRoom = settingsPtr->flag("Vincia:MCoptimization");
  iRegDependence   = true;
  if (uncertaintyBands) {
    optimizeHeadRoom = false;
    if (helicityShower) initHeadRoom(4.0, 4.8, 4.0, 4.0, 4.0, 4.0);
    else initHeadRoom(2.0, 2.4, 2.0, 2.0, 2.0, 2.0);
  } else initHeadRoom(1.0, 1.2, 1.0, 1.0, 1.0, 1.0);
  pAcceptFuss      = settingsPtr->parm("Vincia:PacceptFuss");
  pAcceptPT0       = settingsPtr->parm("Vincia:PacceptPT0");
  nReportMCviol    = 5;
  nReportedMCviol  = 0;

  // Resize Paccept to the maximum number of elements
  Paccept.resize(max(weightsPtr->nVariations(),1));
  PME = 0.0;

  // Hard system(s) and MEclass
  hardSys.clear();
  hardSys.resize(0);
  MEclass.clear();
  MEclass.resize(0);

  // Statistics
  nTrialsSum         = 0;
  nTrialsVeto        = 0;
  nTrialsHull        = 0;
  nTrialsHullMassive = 0;
  nTrialsCutM        = 0;
  nTrialsCutQ        = 0;
  nBornVeto          = 0;
  nBornPlusJetVeto   = 0;
  nClosePSforHQ      = 0;
  nForceSplit        = 0;
  nMCviols.clear();
  nMCviols.resize(3);
  maxMCviols.clear();
  maxMCviols.resize(3);
  nNegPas.clear();
  nNegPas.resize(3);
  minNegPas.clear();
  minNegPas.resize(3);
  for (int i=0; i<3; i++) {
    nMCviols[i]   = 0;
    maxMCviols[i] = 0.0;
    nNegPas[i]    = 0;
    minNegPas[i]  = 0.0;
  }

  // Rescue Levels
  doRescue  = true;
  nRescue   = 100;
  rescueMin = 1.0e-6;

  // Initialize factorization scale and parameters for shower starting scale
  q2fac          = 0.0;
  qFac           = 0.0;
  qMaxMatch      = settingsPtr->mode("Vincia:QmaxMatch");
  q2maxFudge     = pow2(settingsPtr->parm("Vincia:QmaxFudge"));
  q2maxFudgeMPI  = pow2(settingsPtr->parm("Vincia:QmaxFudgeMPI"));

  // Parameters for merging
  string mProc   = settingsPtr->word("Merging:Process");
  doMergeSav     = !(mProc.compare("void") == 0);
  incomingType   = 0;
  if (doMergeSav) {
         if (mProc.find("e+e->")   != string::npos) incomingType = 1;
    else if (mProc.find("mu+mu->") != string::npos) incomingType = 2;
    else if (mProc.find("ta+ta->") != string::npos) incomingType = 3;
    else if (mProc.find("pp>")     != string::npos) incomingType = 10;
  } else {
    int idBeamA = abs(settingsPtr->mode("Beams:idA"));
    int idBeamB = abs(settingsPtr->mode("Beams:idB"));
         if ( (idBeamA == 11  ) && (idBeamB == 11  ) ) incomingType = 1;
    else if ( (idBeamA == 13  ) && (idBeamB == 13  ) ) incomingType = 2;
    else if ( (idBeamA == 15  ) && (idBeamB == 15  ) ) incomingType = 3;
    else if ( (idBeamA == 2212) && (idBeamB == 2212) ) incomingType = 10;
  }

  // Reset saved particle state
  indexSav.resize(0);
  partsSav.resize(0);

  // Initialize USR antennae. Not done before to make sure we catch if
  // something was changed in Settings before.
  if (verbose >= 6)
    cout<<" SpaceAntShower(): initializing antennaSet"<<endl;
  antSetPtr->init();

  // Print VINCIA header and list of parameters
  if (verbose >= 1 && !headerIsPrinted) header();

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::init","end --------------");

  // Is initialized now
  isInit = true;

}

//*********
  
// Limition of first emission.

bool SpaceAntShower::limitPTmax(Event& event, double, double) {
  // We handle scale internally, unless merging is applied.
  if (!doMergeSav) return false;
  // Leptonic initial states
  if ( event[1].isLepton() && event[2].isLepton() ) {
    int nFSparton = 0;
    for (int i=1; i<(int)event.size(); i++) if (event[i].isFinal()) {
      int id = event[i].idAbs();
      if ( (id == 21) || (id < 6) ) nFSparton++;
    }
    if (nFSparton == 2) return true;
  }
  // Hadronic initial states
  if ( event[1].isHadron() && event[2].isHadron() ) {
    int  nFSparton = 0;
    bool isQCDnow  = true;
    for (int i=3; i<(int)event.size(); i++) {
      int id = event[i].idAbs();
      if ( (id != 21) && (id > 5) ) isQCDnow = false;
      else if (event[i].isFinal()) nFSparton++; 
    } 
    if (isQCDnow && (nFSparton == 2)) return true;
    if (nFSparton == 0) return true;
  }
  return false;
}

//*********
  
// Prepare system for evolution
  
void SpaceAntShower::prepare( int iSys, Event& event, bool limitPTmaxIn) {
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::prepare","begin --------------");
    
  // If verbose, list event before showering
  if (verbose >= 4) {
    if (verbose >= 5) event.list();
    printOut("SpaceAntShower::prepare","preparing system no. "
      + num2str(iSys) + " for showering. Size = "
      + num2str(partonSystemsPtr->sizeAll(iSys)) +
      " limitPTmax = " + bool2str(limitPTmaxIn));
  }
    
  // Counter for accepted events
  nAccepted = max(nAccepted, infoPtr->nAccepted());
    
  // First time prepare is called for an event
  bool firstCallInEvent = (iSys == 0);
  // getCounter(3) number of times a Pythia::next() call has been begun.
  int nCallPythiaNextNow = infoPtr->getCounter(3);
  if (nCallPythiaNextNow > nCallPythiaNext) {
    nCallPythiaNext  = nCallPythiaNextNow;
    nVetoUserHooks   = 0;
    nFailHadLevel    = 0;
    firstCallInEvent = true;
  }
  // Last event got accepted.
  long nSelectedNow = infoPtr->nSelected();
  if (nSelectedNow > nSelected) {
    nSelected        = nSelectedNow;
    nVetoUserHooks   = 0;
    nFailHadLevel    = 0;
    firstCallInEvent = true;
  }
  // getCounter(10) number of times the selection of a new hard process has
  // been begun. = 1 if no user hooks veto, > 1 if user hooks veto.
  int nVetoUserHooksNow = (infoPtr->getCounter(10)-1);
  if (nVetoUserHooksNow > nVetoUserHooks) {
    nVetoUserHooks   = nVetoUserHooksNow;
    firstCallInEvent = true; 
  }
  // getCounter(14) number of times the loop over parton- and hadron-level processing 
  // has begun for a hard process. = 1 if everything after user hooks veto is
  // succesfull, > 1 eg if hadron level fails.
  int nFailHadLevelNow = (infoPtr->getCounter(14)-1);
  if (nFailHadLevelNow > nFailHadLevel) {
    nFailHadLevel    = nFailHadLevelNow;
    firstCallInEvent = true;
  }
    
  // Resetting for first time in new event
  if (firstCallInEvent) {
    //if (iSys == 0) {
    // Reset counters
    vinComPtr->resetCounters();

    // Reset the weights in new events
    weightsPtr->resetWeights(infoPtr->nAccepted());

    // Reset all vectors for the first time we are called
    hardSys.clear();          hardSys.resize(0);
    MEclass.clear();          MEclass.resize(0);
    nQLneeded.clear();        nQLneeded.resize(0);
    Hint.clear();             Hint.resize(0);
    additionalJets.clear();   additionalJets.resize(0);
    indexSav.clear();         indexSav.resize(0);
    partsSav.clear();         partsSav.resize(0);
    polarizedSys.clear();     polarizedSys.resize(0);
    branchelementals.clear(); branchelementals.resize(0);
    Q2hat.clear();            Q2hat.resize(0);
    nBranch.clear();          nBranch.resize(0);
    nBranchISR.clear();       nBranchISR.resize(0);
    nG.clear();               nG.resize(0);
    nQQ.clear();              nQQ.resize(0);
    initialA.clear();         initialA.resize(0);
    initialB.clear();         initialB.resize(0);
    eAused = 0.0;
    eBused = 0.0;

    // Evolution windows, add factorization scale
    regMinScalesNow.clear();
    regMinScalesNow = regMinScalesMtSav;
    regMinScalesNow.push_back(sqrt(infoPtr->Q2Fac()));
    // Sort to put factorization scale at the right place
    stable_sort(regMinScalesNow.begin(), regMinScalesNow.end());
    // Fill the rest
    double qMinOld = regMinScalesNow[(int)regMinScalesNow.size()-1];
    double qMinNow = 2.0*qMinOld;
    double multFac = 2.0;
    double sqrtS   = sqrt(sbbSav);
    while (qMinNow < sqrtS) {
      int iRegNew = int( log(qMinNow/qMinOld)/log(6) + 6);
      int iRegMax = (int)regMinScalesNow.size()-1;
      if (iRegNew > iRegMax) 
        regMinScalesNow.push_back( pow(6,(double)(iRegMax+1)-6.0)*qMinOld );
      qMinNow *= multFac;
    }
  }

  // Statistics (zero info on qBranch scales)
  if (iSys < 4) {
    for (int j=0; j<11; ++j) {
      qBranch[iSys][j]=0.0;
      pTphys[iSys][j]=0.0;
    }
  }
    
  // Sanity check: at least two particles in system
  int sizeSystem = partonSystemsPtr->sizeAll(iSys);
  if (sizeSystem <= 1) return;
    
  // Resize to make sure we can access the iSys'th element
  int currentSize = branchelementals.size();
  if ( currentSize < (iSys + 1) ) {
    hardSys.resize(iSys + 1);
    MEclass.resize(iSys + 1);
    nQLneeded.resize(iSys + 1);
    Hint.resize(iSys + 1);
    additionalJets.resize(iSys + 1);
    indexSav.resize(iSys + 1);
    partsSav.resize(iSys + 1);
    polarizedSys.resize(iSys + 1);
    branchelementals.resize(iSys + 1);
    Q2hat.resize(iSys + 1);
    nBranch.resize(iSys + 1);
    nBranchISR.resize(iSys + 1);
    nG.resize(iSys + 1);
    nQQ.resize(iSys + 1);
    initialA.resize(iSys + 1);
    initialB.resize(iSys + 1);
  }
    
  // We don't have a starting scale for this system yet
  Q2hat[iSys] = 0.0;
  // After prepare we always have zero branchings
  nBranch[iSys] = 0;
  nBranchISR[iSys] = 0;
  // Initialize polarization with false or FSR value if possible
  polarizedSys[iSys] = ( ((int)FSRshowerPtr->polarizedSys.size() > iSys) ?
    FSRshowerPtr->polarizedSys[iSys] : false );

  if (verbose >= 9) cout<<" Making colour maps "<<endl;
  
  // Find and save all colors and anticolors
  map<int,int> indexOfAcol; // Put initial colours here
  map<int,int> indexOfCol;  // Put initial anticolours here
  // This way we can always check for the colour in Acol to find antennae
  vector< pair<int,int> > antsISR;
    
  // Loop over event record. Find indices of particles with color lines
  // Make new copies so any modifications by VINCIA are clearly recorded
  for (int i = 0; i < sizeSystem; ++i) {    
    int i1 = partonSystemsPtr->getAll( iSys, i);
    if ( i1 <= 0 ) continue;
    // Save to colour maps
    int col  = event[i1].col();
    int acol = event[i1].acol();
    if (!event[i1].isFinal()) { // Switch colours for initial partons
      col  = acol;
      acol = event[i1].col();
      if (event[i1].pz() > 0.0) {
        initialA[iSys] = event[i1];
        eAused        += event[i1].e();
      } else {
        initialB[iSys] = event[i1];
        eBused        += event[i1].e();
      }
    }
    if (col > 0) indexOfCol[col] = i1;
    else if (col < 0) indexOfAcol[-col] = i1;
    if (acol > 0) indexOfAcol[acol] = i1;
    else if (acol < 0) indexOfCol[-acol] = i1;
    // Update list of initial-initial and initial-final connections
    if (col > 0 && indexOfAcol.count(col) == 1)
      antsISR.push_back( make_pair(i1,indexOfAcol[col]) );
    if (acol < 0 && indexOfAcol.count(-acol) == 1)
      antsISR.push_back( make_pair(i1,indexOfAcol[-acol]) );
    if (acol > 0 && indexOfCol.count(acol) == 1)
      antsISR.push_back( make_pair(indexOfCol[acol], i1) );
    if (col < 0 && indexOfCol.count(-col) == 1)
      antsISR.push_back( make_pair(indexOfCol[-col], i1) );
    // Add to list of particles
    indexSav[iSys].push_back( i1 );
    partsSav[iSys].push_back( event[i1] );
  }

  if (verbose >= 9) cout<<" Checking process class for MECs"<<endl;
  
  // Check the process class for MECs and whether system is "hard"
  bool hardSysNow = false;
  int nPartsTot   = (int)partsSav[iSys].size();
  int nColIn = 0, nColOut = 0, nGluTot = 0, nBosIn = 0, nBosOut = 0;
  int nZandW = 0, iHiggs = -1;
  // Also check if at least one QCD parton in final state (used when setting
  // upper limit for evolution)
  bool hasFSpartons = false;
  for (int ip=0; ip<nPartsTot; ip++) {
    // Is this a coloured object?
    int colSum = partsSav[iSys][ip].col()+partsSav[iSys][ip].acol();
    if (colSum > 0) partsSav[iSys][ip].isFinal() ? nColOut++ : nColIn++;
    // Is this a boson we have in the MECs?
    else if ( (partsSav[iSys][ip].idAbs() > 22) && 
      (partsSav[iSys][ip].idAbs() < 26) ) {
      partsSav[iSys][ip].isFinal() ? nBosOut++ : nBosIn++;
      partsSav[iSys][ip].idAbs() == 25 ? (iHiggs = ip) : nZandW++;
    }
    // Is this a gluon?
    if (partsSav[iSys][ip].id() == 21) nGluTot++;
    // If status code is 21,22,23 we are in the hard system
    if ( abs(partsSav[iSys][ip].statusAbs()-22) < 2 ) hardSysNow = true;
    // If parton is final-state and can be produced by shower? 
    if ( partsSav[iSys][ip].isFinal() ) { 
      int idAbs = partsSav[iSys][ip].idAbs();
      if ( (idAbs >= 1 && idAbs <= 5) || idAbs == 21) hasFSpartons = true;
    }
  }
  // Ask FSR if the system is hard, if so adjust hardSysNow
  // NOTE this needs to be done if FSR made q's massless
  if (FSRshowerPtr->isSysHard(iSys)) hardSysNow = true;
  int nColTot = nColIn + nColOut;
  int nBosTot = nBosIn + nBosOut;
  // Find process class for MG4 MECs, adjust number of branchings (used for 
  // finding QCD order), adjust number of additional jets present
  int MEclassNow   = -1;
  meCorrectionsNow = meCorrections;
  if ( (nColTot == nPartsTot) && (nColOut > 0) && (nColIn > 0) ) {
    MEclassNow     = pureQCD;
    nBranch[iSys]  = nPartsTot-4;
  } else if ( (nBosTot == 1) && (nBosTot+nColTot == nPartsTot) ) {
    MEclassNow     = (nBosOut == 1 ? singleBosonProd : singleBosonDecay);
    nBranch[iSys]  = nColTot-2;
  } else meCorrectionsNow = false;
  // For merging only allow MECs in ordered events
  if (doMergeSav) {
    if (vinMergingHooksPtr->doOrderedMECs()) meCorrectionsNow = meCorrections;
    else meCorrectionsNow = false;
  }
  // Set the properties needed for MECs
  hardSys[iSys]        = hardSysNow;
  MEclass[iSys]        = MEclassNow;
  nQLneeded[iSys]      = nZandW;
  additionalJets[iSys] = nBranch[iSys];
  // For Higgs check if we should do interference
  if (iHiggs > -1 && !Hinterference) {
    Particle higgs = partsSav[iSys][iHiggs];
    int idSum = -999;
    if (MEclassNow == singleBosonDecay)
      idSum = event[higgs.daughter1()].id() + event[higgs.daughter2()].id();
    else if (MEclassNow == singleBosonProd)
      idSum = event[higgs.mother1()].id() + event[higgs.mother2()].id();
    if (idSum > -999) Hint[iSys] = ( (idSum == 42) ? 2 : 1 );
    else Hint[iSys] = 0;
  } else Hint[iSys] = 0;

  // Factorization scale
  if (hardSysNow) {
    q2fac = infoPtr->Q2Fac();
    qFac  = sqrt(q2fac);
  }

  // Set ISR starting scale for this system
  if (hardSysNow) {
    if (verbose >= 9) cout<<" Setting ISR starting scale for hard system"<<endl;
    // Set the initial value of the starting scale: factorisation scale or
    // phase-space maximum according to option
    if (qMaxMatch == 1 || (qMaxMatch == 0 && hasFSpartons) ) {
      // Wimpy: enhanceq2fac * factorisation scale from PYTHIA
      Q2hat[iSys] = q2maxFudge * q2fac;
    } else {
      // Power: Phase-space maximum 
      Q2hat[iSys] = m2( beamAPtr->p(), beamBPtr->p());
    }
    // TMP!
    // Could add an option to let VINCIA calculate its own preferred scale,
    // possibly including a PDF ratio to re-normalise the cross section
    // to use that value.
  } else {
    if (verbose >= 9) cout<<" Setting properties of MPI system"<<endl;
    // Set starting scale for MPI systems: min of incoming parton scales.
    // Find positions of incoming colliding partons.
    int in1 = partonSystemsPtr->getInA(iSys);
    int in2 = partonSystemsPtr->getInB(iSys);
    Q2hat[iSys] = q2maxFudgeMPI * pow2(min(event[in1].scale(),event[in2].scale()));
  }

  // Verbose output
  if (verbose >= 9) cout<<" Q2fac from infoPtr = "<<q2fac<<" -> QF = "<<qFac
                        <<"; Q2start for this system = "<<Q2hat[iSys]
                        <<" -> Qstart = "<<sqrt(Q2hat[iSys])<<endl;

  // If we got non-hard system, renew all trials since PDFs
  // changed and set the restart scales accordingly
  if (!hardSysNow) {
    if (verbose >= 7)
      printOut("SpaceAntShower::prepare",
        "Renewing all trials since we got non-hard system!");
    for (int iSystNow = 0; iSystNow < (int)branchelementals.size(); iSystNow++) 
      if (iSystNow != iSys) {
        for (int iAnt = 0; iAnt < (int)branchelementals[iSystNow].size(); ++iAnt) {
          branchelementals[iSystNow][iAnt].renewTrial();
        }
      }
  }

  // Kick out all FF connections
  int nAntsISR = (int)antsISR.size();
  for (int iAnt=0; iAnt<(int)antsISR.size(); iAnt++)
    if (event[antsISR[iAnt].first].isFinal() && event[antsISR[iAnt].second].isFinal())
      nAntsISR--;

  // Sanity check: can only shower systems with more than 0 II/IF connection
  if (nAntsISR <= 0) return;

  // VINCIA starts by creating a copy of the particles it will work with
  // Colour and polarization information may be changed or added,
  // respectively, and masses may be set to zero for partons VINCIA
  // wants to treat as massless. 
  treatAsMassless = true;
  vector<int> jMakeMassless;
  int nISmassless   = 0;
  int nISmassive    = 0;
  int nFS           = 0;
  for (int i=0; i<(int)indexSav[iSys].size(); i++) {
    int nr = indexSav[iSys][i];
    if ( event[nr].m2Calc() > TINYMASS ) {
      // Everyone with colors gets mapped to massless for ISR
      if (abs(event[nr].col())+abs(event[nr].acol()) > 0)
        jMakeMassless.push_back(i);
      if (!event[nr].isFinal()) nISmassive++;
    }
    else if (!event[nr].isFinal()) nISmassless++;
    if (event[nr].isFinal()) nFS++;
  }
  // Special case, 2->n, with only massless incoming guys
  bool doSpecial    = ( (nISmassive == 0) && (nISmassless = 2) );
  int  nToMassless  = (int)jMakeMassless.size();
  bool madeMassless = (jMakeMassless.size() > 0);
    
  if (verbose >= 9) cout<<" Sanitising input "<<endl;

  // Sanitize input. 
  // Bottom and lighter quarks are always bookkept as massless by Vincia
  // (Top quarks may be made massless as well, for ultra-high energies)
  while (jMakeMassless.size() > 0) {
    // Special case, 2->n, with only massless incoming guys and either
    // 2 partons to make massless or >1 FS guys with 1 to make massless
    if ( doSpecial && ( (nToMassless == 2) || 
        (nToMassless == 1 && nFS > 1) ) ) {
      int i1 = jMakeMassless[0];
      int i2 = jMakeMassless[nToMassless-1];
      if (i2 == i1) { // Need massive partner
        for (int ip=0; ip<(int)partsSav[iSys].size(); ip++) {
          if (partsSav[iSys][ip].isFinal() && (ip != i1)) i2 = ip;
        }
      }
      // Go to rest frame of pair
      Vec4 p1 = partsSav[iSys][i1].p();
      Vec4 p2 = partsSav[iSys][i2].p();
      RotBstMatrix M;
      M.toCMframe(p1,p2);
      p1.rotbst(M);
      p2.rotbst(M);
      double eCMs = p1.e() + p2.e();
      // Check if both to be massless
      if (nToMassless == 2 || partsSav[iSys][i2].m2() <= TINY) {
        p1 = Vec4(0.0, 0.0,  eCMs/2.0, eCMs/2.0);
        p2 = Vec4(0.0, 0.0, -eCMs/2.0, eCMs/2.0);
        // PYTHIA stores mass as "5th" component, to preserve precision
        partsSav[iSys][i1].m(0.0);
        partsSav[iSys][i2].m(0.0);
      }
      // Else just make i1 massless
      else {
        double e1 = (eCMs - partsSav[iSys][i2].m2()/eCMs)/2.;
        double e2 = eCMs - e1;
        p1 = Vec4(0.0, 0.0,  e1, e1);
        p2 = Vec4(0.0, 0.0, -e1, e2);
        // PYTHIA stores mass as "5th" component, to preserve precision
        partsSav[iSys][i1].m(0.0);
      }
      M.invert();
      p1.rotbst(M);
      p2.rotbst(M);
      partsSav[iSys][i1].p(p1);
      partsSav[iSys][i2].p(p2);
      jMakeMassless.clear();
      jMakeMassless.resize(0);
      continue;
    }
    // Special case, 2->n, with only massless incoming guys
    // use mapToMassless
    if (doSpecial) {
      vector<Vec4> momenta;
      for (int i=0; i<(int)jMakeMassless.size(); i++)
        momenta.push_back(partsSav[iSys][jMakeMassless[i]].p());
      // Mapping
      vinCluPtr->mapToMassless(momenta, 0);
      // Set new momenta
      for (int i=0; i<(int)jMakeMassless.size(); i++) {
        partsSav[iSys][jMakeMassless[i]].p(momenta[i]);
        // PYTHIA stores mass as "5th" component, to preserve precision
        partsSav[iSys][jMakeMassless[i]].m(0.0);
      }
      jMakeMassless.clear();
      jMakeMassless.resize(0);
      continue;
    }
    // Start with last parton
    // (-> easy to use pop_back to erase it when we are done with it)
    int i1 = jMakeMassless[jMakeMassless.size()-1];
    Particle& parton1 = partsSav[iSys][i1];
    // If this particle was already made massless, erase and continue
    if (parton1.m2() <= TINY) {
      jMakeMassless.pop_back();
      continue;
    }
    // Just make this parton massless
    if (parton1.idAbs() > 6) {
      double facInv = parton1.pAbs()/parton1.e();
      if (facInv > 0.0) parton1.rescale3(1./facInv);
      // PYTHIA stores mass as "5th" component, to preserve precision
      partsSav[iSys][i1].m(0.0);
      partsSav[iSys][i1].p(parton1.p());
    } else { // Find matching partner
      bool isIn1  = (!parton1.isFinal());
      bool isAnti = (parton1.id() < 0);
      int col1 = (isAnti ? parton1.acol() : parton1.col());
      for (int i2=0; i2<(int)partsSav[iSys].size(); i2++) {
        // Skip self
        if (i2 == i1) continue;
        Particle& parton2 = partsSav[iSys][i2];
        bool isIn2  = (!parton2.isFinal());
        // Look for opposite-colour 
        int col2 = ( ((isIn1 && isIn2) || (!isIn1 && !isIn2)) ? 
          (isAnti ? parton2.col() : parton2.acol()) :
          (isAnti ? parton2.acol() : parton2.col()) );
        if (col1 == col2) {
          // Go to rest frame of parton pair
          Vec4 p1 = parton1.p();
          Vec4 p2 = parton2.p();
          RotBstMatrix M;
          M.toCMframe(p1,p2);
          p1.rotbst(M);
          p2.rotbst(M);
          double eCMs = p1.e() + p2.e();
          // Check if both to be massless
          if (parton2.m2() <= TINY) {
            p1 = Vec4(0.0, 0.0,  eCMs/2.0, eCMs/2.0);
            p2 = Vec4(0.0, 0.0, -eCMs/2.0, eCMs/2.0);
            // PYTHIA stores mass as "5th" component, to preserve precision
            partsSav[iSys][i1].m(0.0);
            partsSav[iSys][i2].m(0.0);
          }
          // Else just make this parton massless
          else {
            double e1 = (eCMs - parton2.m2()/eCMs)/2.;
            double e2 = eCMs - e1;
            p1 = Vec4(0.0, 0.0,  e1, e1);
            p2 = Vec4(0.0, 0.0, -e1, e2);
            // PYTHIA stores mass as "5th" component, to preserve precision
            partsSav[iSys][i1].m(0.0);
          }
          M.invert();
          p1.rotbst(M);
          p2.rotbst(M);
          partsSav[iSys][i1].p(p1);
          partsSav[iSys][i2].p(p2);
          break;
        }
      }
    }
    // Done with this parton
    jMakeMassless.pop_back();
  }
    
  // Polarization: check if VINCIA should (attempt to) assign helicities
  bool doPolarize = false;
  usePolarizationNow = (treatAsMassless && helicityShower);
  if (usePolarizationNow) {
    // Check whether event is already polarized, allow bosons to be unpolarized
    bool isPolarized = true;
    for (int i=0; i<(int)partsSav[iSys].size(); ++i) 
      if ( !(partsSav[iSys][i].idAbs() > 21 && partsSav[iSys][i].idAbs() < 38)
        && partsSav[iSys][i].pol() == 9 ) isPolarized = false;
    // If not, see if we can polarize it
    if (!isPolarized) {
      matrixElementCorrectionsPtr->prepMECs(MEclassNow, nZandW, Hint[iSys]);
      usePolarizationNow = 
        matrixElementCorrectionsPtr->polarize(2,partsSav[iSys]);
      if (usePolarizationNow) doPolarize = true;
    }
  }
  // Set system flag and update general flag
  polarizedSys[iSys] = usePolarizationNow;
  if ((int)FSRshowerPtr->polarizedSys.size() > iSys)
    FSRshowerPtr->polarizedSys[iSys] = usePolarizationNow;
  usePolarizationNow = (treatAsMassless && helicityShower);
  if (verbose >= 7) printOut("SpaceAntShower::prepare",
    "doPolarize = "+bool2str(doPolarize));
    
  // Assign each dipole a random color index between 1 and 9.
  // (For use in colour reconnections)
  bool doColorize = (CRmode >= 1);
    
  // Create new (polarized, massless, color-indexed) copies of
  // original parton system;
  // only if we changed anything to avoid with changing the
  // system after FSR already did
  if (madeMassless || doPolarize || doColorize) {

    if (verbose >= 9) cout<<" Creating copies of input particles "<<endl;

    map<int,int> iChanged;
    indexOfCol.clear();
    indexOfAcol.clear();
    for (int i=0; i<(int)partsSav[iSys].size(); i++) {
      int iOld = indexSav[iSys][i];
      int iNew = iOld;
      if (event[iOld].isFinal()) iNew = event.copy(iOld,43);
      else {
        iNew = event.copy(iOld,-41);
        // Everyone who has iOld as daughter needs to get
        // iNew as daughter now
        for (int ip=0; ip<(int)event.size(); ip++) if (ip < iOld) {
            if (event[ip].daughter1() == iOld) {
              // iNew > daughter2, that's fine
              event[ip].daughters(iNew, event[ip].daughter2());
            }
            if (event[ip].daughter2() == iOld) {
              // iNew < daughter2, need swap
              event[ip].daughters(iNew, event[ip].daughter1());
            }
          }
        // Update the beam pointers
        BeamParticle& beamNow = (event[iNew].pz() > 0.0 ? *beamAPtr : *beamBPtr);
        double eBeamNow       = (event[iNew].pz() > 0.0 ? eBeamA : eBeamB);
        beamNow[iSys].update( iNew, event[iNew].id(), event[iNew].e()/eBeamNow );
      }
      iChanged[iOld] = iNew;
      // Update 4-momentum and mass (eg if made massless)
      event[iNew].p(partsSav[iSys][i].p());
      event[iNew].m(partsSav[iSys][i].m());
      // Set polarizations and colors if new ones defined
      if (polarizedSys[iSys]) event[iNew].pol(partsSav[iSys][i].pol());
      // Update properties in indexSav[iSys] and partsSav[iSys]
      indexSav[iSys][i] = iNew;
      partsSav[iSys][i] = event[iNew];
      // Update the list of systems.
      partonSystemsPtr->replace(iSys, iOld, iNew);
      // Update color maps
      int col  = event[iNew].col();
      int acol = event[iNew].acol();
      if (!event[iNew].isFinal()) { // Switch colours for initial partons
        col  = acol;
        acol = event[iNew].col();
      }
      if (col  != 0) indexOfCol[col]   = iNew;
      if (acol != 0) indexOfAcol[acol] = iNew;
    }
    // Let FSR know we did changes to the system
    FSRshowerPtr->updateAfterISR(iSys, event, iChanged);
  }
    
  if (verbose >= 6) {
    printOut("SpaceAntShower::prepare", "saved the following "
      "particles: nr id col acol p");
    for (int i=0; i<(int)partsSav[iSys].size(); i++) {
      cout << "  " << setw(3) << indexSav[iSys][i];
      cout << " " << setw(2) << partsSav[iSys][i].id();
      cout << " " << setw(3) << (int)partsSav[iSys][i].pol();
      cout << " " << setw(3) << partsSav[iSys][i].col();
      cout << " " << setw(3) << partsSav[iSys][i].acol();
      cout << " " << partsSav[iSys][i].p();
    }
  }
    
  // Now loop over colored particles to create branchelementals (=antennae)
  for (map<int,int>::iterator it=indexOfCol.begin(); 
       it != indexOfCol.end(); ++it) {
    int col  = it->first;
    int i1   = it->second;
    int i2   = indexOfAcol[col];
    if (col == 0 || i1 == 0 || i2 == 0) continue;
    // Exclude final-final antennae
    if ( (event[i1].isFinal()) && (event[i2].isFinal()) ) continue;
    if (verbose >= 7) {
      printOut("SpaceAntShower::prepare",
        "Creating antenna between "+num2str(i1,4)+", "
        +num2str(i2,4)+": col = "+num2str(col,5));
    }
      
    BeamParticle& beam1   = (event[i1].pz() > 0.0) ? *beamAPtr : *beamBPtr;
    BeamParticle& beam2   = (event[i2].pz() > 0.0) ? *beamAPtr : *beamBPtr;
    bool isVal1           = beam1[iSys].isValence();
    bool isVal2           = beam2[iSys].isValence();
      
    // Store trial QCD antenna and add trial generators depending on type
    BranchelementalISR trial(event,i1,i2,isVal1,isVal2);
    resetTrialGenerators(&trial);
      
    // Save system
    trial.system = iSys;
      
    trial.col = col;
    trial.renewTrial();
    trial.mCalc();
      
    // Updates to find neighbours
    i1 = trial.iOld1; // If II always p+ guy. if IF always I guy
    i2 = trial.iOld2; // If II always p- guy, if IF always F guy
    int col1  = event[i1].acol(); // 1 is always initial
    int acol1 = event[i1].col();
    int col2  = ( event[i2].isFinal() ? event[i2].col() : event[i2].acol() );
    int acol2 = ( event[i2].isFinal() ? event[i2].acol() : event[i2].col() );
    // Is it the color of 1 that they share?
    bool shareCol1 = ( (trial.col==col1) ? true : false );
    if (shareCol1) {
      // Neighbor on i1 side
      if (acol1 != 0) trial.iL = indexOfCol[acol1];
      // Neighbor on i2 side
      if (col2 != 0) trial.iR = indexOfAcol[col2];
    }
    else {
      // Neighbor on i1 side
      if (col1 != 0) trial.iL = indexOfAcol[col1];
      // Neighbor on i2 side
      if (acol2 != 0) trial.iR = indexOfCol[acol2];
    }
      
    branchelementals[iSys].push_back(trial);
  }
    
  // Count up number of gluons and quark pairs
  nG[iSys]  = 0;
  nQQ[iSys] = 0;
  for (int i=0; i<(int)partsSav[iSys].size(); i++) {
    if (partsSav[iSys][i].id()==21) nG[iSys]++;
    else if (abs(partsSav[iSys][i].id()) < 7) nQQ[iSys]++;
  }
  // Half the quarks to get quark pairs
  nQQ[iSys] = nQQ[iSys]/2;

  // Sanity check
  if (branchelementals[iSys].size() <= 0) {
    if (verbose >= 5) printOut("SpaceAntShower::prepare",
      "did not find any antennae: exiting.");
    return; 
  }
      
  //If verbose output, list dipoles
  if (verbose >= 5) list(); 
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::prepare","finished --------------");
    
}
  
//*********
  
// Update list of branchelementals after FSR radiation
  
void SpaceAntShower::update( int iSys, Event& event, bool) {
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::update","begin --------------");
    
  // If verbose, list event
  if (verbose >= 5) event.list();
    
  // Count number of branches
  nBranch[iSys]++;
    
  // Particles in the list are already updated by FSR
  // Find and save all colors and anticolors
  map<int,int> indexOfAcol; // Put initial colours here
  map<int,int> indexOfCol;  // Put initial anticolours here
  int sizeSystem = partonSystemsPtr->sizeAll(iSys);
  for (int i = 0; i < sizeSystem; ++i) {    
    int i1 = partonSystemsPtr->getAll( iSys, i);
    if ( i1 <= 0 ) continue;
    // Save to colour maps
    int col  = event[i1].col();
    int acol = event[i1].acol();
    if (!event[i1].isFinal()) { // Switch colours for initial partons
      col  = acol;
      acol = event[i1].col();
    }
    if (col > 0) indexOfCol[col] = i1;
    else if (col < 0) indexOfAcol[-col] = i1;
    if (acol > 0) indexOfAcol[acol] = i1;
    else if (acol < 0) indexOfCol[-acol] = i1;
  }
    
  // Count up number of gluons and quark pairs
  nG[iSys]  = 0;
  nQQ[iSys] = 0;
  for (int i=0; i<(int)partsSav[iSys].size(); i++) {
    if (partsSav[iSys][i].id()==21) nG[iSys]++;
    else if (abs(partsSav[iSys][i].id()) < 7) nQQ[iSys]++;
  }
  // Half the quarks to get quark pairs
  nQQ[iSys] = nQQ[iSys]/2;
    
  if (verbose >= 6) {
    printOut("SpaceAntShower::update", "saved the following "
      "particles: nr id col acol p");
    for (int i=0; i<(int)partsSav[iSys].size(); i++) {
      cout << "  " << setw(3) << indexSav[iSys][i];
      cout << " " << setw(2) << partsSav[iSys][i].id();
      cout << " " << setw(3) << partsSav[iSys][i].col();
      cout << " " << setw(3) << partsSav[iSys][i].acol();
      cout << " " << partsSav[iSys][i].p();
    }
  }
    
  // Loop over the antennae, we only need to check the IF ones,
  // since we are the only one to change initial partons
  // Neighbours to be checked always
  for (int i=0; i<(int)branchelementals[iSys].size(); i++) {
    BranchelementalISR* antPtr = &branchelementals[iSys][i];
    int antCol = antPtr->col;
    if (!antPtr->isII())  {
      // Check if the final guy has still a positive status code ( = is final )
      if (!event[antPtr->iOld2].isFinal()) { // Find the final guy with the colour of the antenna
        int idFold = antPtr->old2.id();
        if (antPtr->old2.col() == antCol) antPtr->iOld2 = indexOfCol[antCol];
        else antPtr->iOld2 = indexOfAcol[antCol];
        antPtr->old2  = event[antPtr->iOld2];
        // Recompute antenna mass
        antPtr->mCalc();
        // Update antPtr generators in case of flavour change
        if (idFold != antPtr->old2.id()) resetTrialGenerators(antPtr);
        // Erase any saved trials
        antPtr->renewTrial();
      }
    }
    // Update the left neighbour
    if (abs(antPtr->old1.colType())==1) antPtr->iL = 0;
    else {
      if (antPtr->old1.col() == antCol) antPtr->iL = indexOfAcol[antPtr->old1.acol()];
      else antPtr->iL = indexOfCol[antPtr->old1.col()];
    }
    // Update the right neighbour
    if (abs(antPtr->old2.colType())==1) antPtr->iR = 0;
    else {
      if (antPtr->isII()) {
        if (antPtr->old2.col() == antCol) antPtr->iR = indexOfAcol[antPtr->old2.acol()];
        else antPtr->iR = indexOfCol[antPtr->old2.col()];
      }
      else {
        if (antPtr->old2.col() == antCol) antPtr->iR = indexOfCol[antPtr->old2.acol()];
        else antPtr->iR = indexOfAcol[antPtr->old2.col()];
      }
    }
  }    
    
  // Sanity check
  if (branchelementals[iSys].size() <= 0) {
    if (verbose >= 5) printOut("SpaceAntShower::update",
      "did not find any antennae: exiting.");
    return; 
  }
      
  // If verbose output, list dipoles
  if (verbose >= 5) list();
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::update","finished --------------");
    
}
     

//*********
  
// Select next pT in downwards evolution of the existing dipoles.
  
double SpaceAntShower::pTnext( Event& event, double pTevolBegAll, 
  double pTevolEndAll, int /*nRadIn*/, bool /*doTrialIn*/) {

  // Sanity checks
  if (branchelementals.size() <= 0) return 0.0;
  if (!settingsPtr->flag("Vincia:ISR")) return 0.0;

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::pTnext","begin --------------");

  // Verbose output
  if (verbose >= 4)
    printOut("SpaceAntShower::pTnext",
      "(re)starting evolution between pTevolBegAll = " + num2str(pTevolBegAll) +
      " and pTevolEndAll = " + num2str(pTevolEndAll));

  // Checks: skipped ants (q[MPI/FSR] > qISR)?, qEnd < all cutoffs?, no ants?
  bool allSkipped = true, qEndsmallerCutoff = true, zeroAnts = true;

  // Denote VINCIA scales by "q", PYTHIA ones by "pTevol".
  double qOld    = pTevolBegAll;    
  double qEndAll = pTevolEndAll;

  // Initialize winner scale
  double qWin = 0.0;
  winnerPtr   = NULL;
  indxWin     = -1;
  iSysWin     = -1;

  // Verbose output
  if (verbose >= 6) list();

  // Loop over the systems we have ----------- BEGIN
  int nrSystems = (int)branchelementals.size();
  for (int iSys = 0; iSys < nrSystems; iSys++) {

    int nrAnts = (int)branchelementals[iSys].size();
    if (nrAnts <= 0) continue;
    zeroAnts = false;

    // Max Q scale for this system
    double qMax = ( doMergeSav ? qOld : min( qOld, sqrt(Q2hat[iSys]) ) );

    // Loop over antennae ----------- BEGIN
    for (int iAnt = 0; iAnt < nrAnts; ++iAnt) {

      // Shorthands for current antenna
      BranchelementalISR* trialPtr = &branchelementals[iSys][iAnt];
      double s12  = trialPtr->s();
      int id1     = trialPtr->old1.id();
      int id2     = trialPtr->old2.id();
      double e1   = trialPtr->e1();
      double e2   = trialPtr->e2();
      bool isII   = trialPtr->isII();
      bool is1A   = trialPtr->is1A();

      // Verbose output
      if (verbose >= 6) {
        printOut("SpaceAntShower::pTnext",
          "* processing antenna " + num2str(iAnt,4)
          + " Sys = " + num2str(iSys,4) + " id1 = " + num2str(id1) + " id2 = "
          + num2str(id2) + " col1 = (" + num2str(trialPtr->old1.col()) + ","
          + num2str(trialPtr->old1.acol()) + ")" + " col2 = ("
          + num2str(trialPtr->old2.col()) + "," + num2str(trialPtr->old2.acol())
          + ")" + " nTrialGens = " + num2str(trialPtr->nTrialGenerators(),2));
      }

      // Generate new trial branchings, starting from qMax
      double qBegin = qMax;

      // Lowest evolution boundary: impose hadronization scale.
      double qEnd = qEndAll;
      // Cutoff scale, depends on whether we are II or IF antenna
      double qCutoff = (isII ? cutoffScaleII : cutoffScaleIF);

      // No trial generators for this antenna
      if (trialPtr->nTrialGenerators() == 0) continue;

      // Loop over trial functions. Find and save max trial scale.
      double qTrialMax = 0.0;
      int indxMax      = -1;

      // Loop over trial generators ----------- BEGIN
      for (int indx = 0; indx < (int)trialPtr->nTrialGenerators(); ++indx) {

        // Pointer to trial generator for this trial:
        TrialGeneratorISR* trialGenPtr = trialPtr->trialGenPtrSav[indx];
        string trialGenName            = trialGenPtr->name();
        int iAntPhys                   = trialPtr->getPhysIndex(indx);
        bool swapped                   = trialPtr->getIsSwapped(indx);

        // Current normalization factor
        double evNormNow = getEvNormFac(iAntPhys);

        double qBeginNow = qBegin;
        double qEndNow   = qEnd;

        // Phase space limit can never be exceeded
        qBeginNow = min(qBeginNow,
          sqrt(trialGenPtr->getQ2max(s12, e1, 
              is1A ? eAused : eBused)));

        // If restart scale < scale we already found continue
        if (qBeginNow < qWin || qBeginNow < qTrialMax) continue;

        // Determine what the hadronization cutoff is for this 
        // paticular combination of antenna and trial generator
        double qCutoffNow = sqrt(evNormNow)*qCutoff;

        if (qEndAll > qCutoffNow) qEndsmallerCutoff = false;

        // Check if any phase space still open
        if ( (qBeginNow <= qCutoffNow) || (qBeginNow <= qEndNow) ) {
          // Verbose output
          if (verbose >= 9)
            printOut("SpaceAntShower::pTnext",
              "skipping this trial since qBeginNow = " + num2str(qBeginNow) +
              " qCutoffNow = " + num2str(qCutoffNow) +
              " qEndNow = " + num2str(qEndNow));
          continue;
        }
        allSkipped = false;

        double qTrial = qBeginNow;

        bool needNewTrial = true;
        // TMP! Saved trials disabled
        /*
        // Check if we have a saved trial, unless we have external merging
        if (!doMergeSav && trialPtr->hasTrial(indx)) {
          if (trialPtr->getTrialScale(indx) < (qBeginNow+TINY)) {
            qTrial = trialPtr->getTrialScale(indx);   
            // Verbose output
            if (verbose >= 9)
              printOut("TimeAntShower::pTnext",
                "Trial " + num2str(indx) + " " + trialGenName +
                " Retrieving saved qTrial = " + num2str(qTrial));
            needNewTrial = false;
          }
        }
        */

        // Generate new q value / no saved trial ----------- BEGIN
        if (needNewTrial || doMergeSav) {

          // Impose evolution windows
          bool acceptRegion = false;
          int iRegion = getRegion(qTrial);

          // Go through regions ----------- BEGIN
          while (!acceptRegion) {

            // Set overestimated Z range for trial generation
            double qMinNow  = getQmin(iRegion);
            double q2MinNow = pow2(qMinNow);
            double zMinNow  = trialGenPtr->getZmin(q2MinNow, s12, e1,
              is1A ? eAused : eBused);
            double zMaxNow  = trialGenPtr->getZmax(q2MinNow, s12, e1,
              is1A ? eAused : eBused);

            // Set headroom factor
            int trialGenInt = trialGenPtr->statInt();
            double headRoomFac = getHeadRoom(trialGenInt, iRegion, nG[iSys], 
                                             nQQ[iSys], iSys, qMinNow);

            // Check for rescue mechanism in case trial gets stuck
            if (doRescue && trialPtr->getNshouldRescue(indx) >= nRescue) {
              // Multiply headroom
              double logRescue = ((double)(trialPtr->getNshouldRescue(indx))) /
                ((double)(nRescue));
              headRoomFac *= pow(10.0,-logRescue);
              if (verbose >= 8)
                printOut("SpaceAntShower::pTnext",
                  "Applying rescue mechanism, nShouldRescue = " + 
                  num2str(trialPtr->getNshouldRescue(indx)) + 
                  ", multiplying headroom with 10^-" + num2str(logRescue));
            }

            // Set PDFratio
            double pdfRatio = 1.0;
            // If II the first is always side A
            // If IF check which side the initial guy is
            BeamParticle* beamAPtrNow = ( (!isII && !is1A) ? beamBPtr :
              beamAPtr );
            // PDF scale
            double PDFscale = pow2(qTrial);
            pdfRatio = trialGenPtr->trialPDFratio(beamAPtrNow, beamBPtr, iSys,
              false, id1, id2, e1, e2, PDFscale, PDFscale);

            // Set color factor for trial
            double colFac = getAnt(iAntPhys)->getChargeFactor();
            int nF        = getNf(iRegion);
            if (iAntPhys == iXGsplitIF) colFac *= min(nF,nGluonToQuarkF);

            // Verbose output
            if (verbose >= 9)
              printOut("SpaceAntShower::pTnext", "Trial " + num2str(indx)
                + " " + trialGenName + ": qBeginNow = " + num2str(qTrial)
                + " (evolution window " + num2str(iRegion) + ", qMin = "
                + num2str(qMinNow) + ", nF = " + num2str(nF) + ") qCutoffNow = "
                + num2str(qCutoffNow) + " qEnd = " + num2str(qEnd));

            // Effective renormalization-scale prefactor. 
            double kR = aSkMu2;
            if (iAntPhys == iQXsplitII || iAntPhys == iQXsplitIF)
              kR = aSkMu2SplitI;
            else if (iAntPhys == iGXconvII || iAntPhys == iGXconvIF)
              kR = aSkMu2Conv;
            else if (iAntPhys == iXGsplitIF) kR = aSkMu2SplitF;
            kR = sqrt(kR);

            // Check if we should use running alphaS
            bool runAlpha = (alphaSorder >= 1);
            // If we are close to lambdaQCD, use constant instead
            if (qMinNow < 2.5*alphaSptr->Lambda3()/kR) runAlpha = false;

            // Check if we should try to get rid of c and b quarks
            // iRegion 2 = mc to mb, iRegion 3 = mb to mtb
            int idCheck = -1;
            if (massCorrectionLevel > 0) {
              // Flavours to check
              for (int i=0; i<(5-nFlavZeroMass); i++) {
                if ( (abs(id1) == 5-i) && (iRegion == 3-i) ) {
                  if ( (iAntPhys == iQXsplitII && !swapped) || 
                    (iAntPhys == iQXsplitIF) ) idCheck = 5-i;
                }
                if ( isII && (abs(id2) == 5-i) && (iRegion == 3-i) ) {
                  if (iAntPhys == iQXsplitII && swapped) idCheck = 5-i;
                }
              }
            }
            bool usePDFmassThreshold = (idCheck > 0);

            // Sanity check for zero branching probability
            if (colFac < TINY || headRoomFac < TINY ) {
              // Save old scale
              double qTmp = qTrial;
              qTrial = 0.0;
              trialPtr->saveTrial(indx,qTmp,qTrial,0.,0.,0.,pdfRatio,0.,0.,0.);
            }

            // 1) Mass treatment, use PDFs mass thresholds with constant alphaS
            else if (usePDFmassThreshold) {
              // Save old scale
              double qTmp = qTrial;
              // Add extra headroom
              headRoomFac *= (iAntPhys == iQXsplitII ? 1.1 : 1.3);
              // Overestimate
              double mu2eff    = mu2min + pow2(kR*qMinNow);
              // alphaS for overestimate
              double facAlphaS = min(alphaSmax, alphaSptr->alphaS(mu2eff));
              if (alphaSorder == 0) facAlphaS = alphaSvalue;
              // Generate new q value, with constant alphaS
              double q2trial = trialGenPtr->genQ2(pow2(qTrial), s12, zMinNow,
                zMaxNow, colFac, facAlphaS, pdfRatio, id1, id2, e1, e2, true,
                headRoomFac);
              qTrial = sqrt(q2trial);
              double massNow = (idCheck == 4 ? mc : mb);
              double extraMassPDFfactor = log(q2trial/pow2(massNow)/evNormNow);
              // trial information
              trialPtr->saveTrial(indx, qTmp, qTrial, zMinNow, zMaxNow, colFac,
                facAlphaS, pdfRatio, 1.0, extraMassPDFfactor, headRoomFac);
              if (verbose >= 9)
                printOut("SpaceAntShower::pTnext", "Using vanishing pdfs "
                  "towards the mass threshold for id1 " + num2str(id1) + 
                   " and id2 " + num2str(id2));
            }

            // 2) AlphaS running inside trial integral
            else if (runAlpha) {
              // Save old scale
              double qTmp = qTrial;
              // One-loop beta function (two-loop imposed by veto, below)
              double b0 = (33.0 - 2.0*nF) / (12.0 * M_PI);
              // Use 3-flavour Lambda for overestimate
              double lambdaEff = alphaSptr->Lambda3();
              // Generate new q value, with alphaS running inside trial integral
              double q2trial = trialGenPtr->genQ2(pow2(qTrial), s12, zMinNow,
                zMaxNow, colFac, pdfRatio, b0, kR, lambdaEff, e1, e2,
                headRoomFac);
              qTrial = sqrt(q2trial);
              // Save trial information
              double muEff    = kR*qTrial/sqrt(evNormNow)/lambdaEff;
              double alphaEff = 1.0/b0/log(pow2(muEff));
              trialPtr->saveTrial(indx, qTmp, qTrial, zMinNow, zMaxNow, colFac,
                alphaEff, pdfRatio, 1.0, 1.0, headRoomFac);
            }

            // 3) AlphaS outside trial integral
            else { 
              // Save old scale
              double qTmp = qTrial;
              // Constant alphaS
              double facAlphaS = ( (alphaSorder >= 1) ? alphaSmax
                                   : alphaSvalue );
              // Generate new q value, with constant alphaS
              double q2trial = trialGenPtr->genQ2(pow2(qTrial), s12, zMinNow,
                zMaxNow, colFac, facAlphaS, pdfRatio, e1, e2, headRoomFac);
              qTrial = sqrt(q2trial);
              // Save trial information
              trialPtr->saveTrial(indx, qTmp, qTrial, zMinNow, zMaxNow, colFac,
                facAlphaS, pdfRatio, 1.0, 1.0, headRoomFac);
            } 

            // Check evolution window boundaries
            if (qTrial > qMinNow) acceptRegion = true;
            else if (qMinNow < qWin || qMinNow < qTrialMax) {
              if (verbose >= 8)
                printOut("SpaceAntShower::pTnext",
                  "stopping evolution, already found scale that is bigger");
              acceptRegion = true;
              trialPtr->renewTrial(indx);
              // TMP! need to set restart scales properly once saved trials
              // will be used!
              qTrial = 0.0;
            } else if (iRegion == 0) { 
              acceptRegion = true;
              qTrial       = 0.0; 
            } else { 
              qTrial = qMinNow;
              iRegion--; 
            }

          }
          // Go through regions ----------- END

        }
        // Generate new q value / no saved trial ----------- END

        if ( (qTrial > qTrialMax) && (qTrial > qCutoffNow) ) {
          qTrialMax = qTrial;
          indxMax   = indx;
        }

        // Check for rescue mechanism in case trial gets stuck
        if ( doRescue && abs(qBeginNow-qTrial) < rescueMin )
          trialPtr->addRescue(indx);

      }
      // Loop over trial generators ----------- END

      // Did I win?
      if (qTrialMax >= qWin || qWin <= 0.0) {      
        winnerPtr = trialPtr;
        qWin      = qTrialMax;
        indxWin   = indxMax;
        iSysWin   = iSys;
      }

      // Verbose output
      if (verbose >= 7)
        printOut("SpaceAntShower::pTnext", "qTrialMax = " + num2str(qTrialMax)
          + (indxMax >= 0 ? " ("+trialPtr->trialGenPtrSav[indxMax]->name()+")"
          : "") + (iAnt+1 == nrAnts ? "; final qWin = " : "; current qWin = ")
          + num2str(qWin) + " i1 i2 = " + num2str(winnerPtr->iOld1) + " "
          + num2str(winnerPtr->iOld2) + " in system " + num2str(iSysWin,4));

    }
    // Loop over antennae ----------- END

  }
  // Loop over the systems we have ----------- END
 
  // If non-zero branching scale found: continue
  if ((qWin > qEndAll) && (qWin > 0.0)) {
    // Verbose output
    if (verbose >= 5) {
      printOut("SpaceAntShower::pTnext", "=== Winner at scale qWin = "
        + num2str(qWin)
        + " trial type " + winnerPtr->trialGenPtrSav[indxWin]->name()
        + " pdf ratio " + num2str(winnerPtr->getPDFratioTrial(indxWin))
        + " col1 = " + num2str(winnerPtr->old1.col())
        + " acol1 = " + num2str(winnerPtr->old1.acol())
        + " col2 = " + num2str(winnerPtr->old2.col())
        + " acol2 = " + num2str(winnerPtr->old2.acol())
        + " in System " + num2str(iSysWin,4));
    }
  }

  else {
    qWin = 0.0;
    // Check if qEnd < all cutoffs
    if (qEndsmallerCutoff && !zeroAnts) {
      // Verbose output
      if (verbose >= 5) {
        printOut("SpaceAntShower::pTnext","=== All trials now below cutoff "
          "qEndAll = "+num2str(qEndAll,3)+".");
        if (verbose >= 6) {
          printOut("SpaceAntShower::pTnext","Final configuration was:");
          event.list();
        }
      }

      // ISR is done (at least for this system), set the weights
      weightsPtr->doWeighting();
    }
  }

  // Check if we have a heavy quark in the initial state left
  bool forceSplitNow = false;
  if ( (!allSkipped) && (massCorrectionLevel > 0) &&
    heavyQuarkLeft(max(qWin,qEndAll)) ) {
    if (verbose >= 6)
      printOut("SpaceAntShower::pTnext",
        "Going to force a splitting! qWin was " + num2str(qWin) +
        " with id1 = " + num2str(winnerPtr->old1.id()) +
        " with id2 = " + num2str(winnerPtr->old2.id()));
    // Set scale to the corresponding mass
    qWin = winnerPtr->scaleSav[indxWin];
    winnerPtr->forceSplittingSav = true;
    forceSplitNow = true;
  } else if (winnerPtr != NULL) winnerPtr->forceSplittingSav = false;

  // End of pTnext: finalize and return to PYTHIA

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::pTnext","finished --------------");

  // Make sure we get the next branching
  if ((qWin > 0.0) && forceSplitNow)
    return (pTevolEndAll-TINY);

  // Done! 
  return qWin;

}
  
//*********
  
// Perform a branching
  
bool SpaceAntShower::branch( Event& event) {
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::branch","begin --------------");
    
  // Count up global number of attempted trials
  ++nTrialsSum;
    
  // System of index of the winner
  int iSys       = iSysWin;
  // Tell Pythia in which system we branched
  iSysSel        = iSys;
  int iTrial     = indxWin;
  // Extract current QE scales
  double qNew    = winnerPtr->getTrialScale(iTrial);
  double q2new   = pow2(qNew);
  int iAntPhys   = winnerPtr->getPhysIndex(iTrial);
  bool isII      = winnerPtr->isII();
  bool is1A      = winnerPtr->is1A();
  bool isSwapped = (isII ? winnerPtr->getIsSwapped(iTrial) : false);
  // Set to false for IF because there it was always guy 1 how did
  // gluon splitting / conversion in the initial state
    
  // Verbose output
  if (verbose >= 5)
    printOut("SpaceAntShower::branch","* Processing Branching at scale Q = " 
      +num2str(qNew));
    
  //------------------------------------------------------------------------
  // RECOILER
  // For II antennae (complete final state)
  vector<Vec4> recoiler; recoiler.clear(); recoiler.resize(0);
  vector<int> iRecs; iRecs.clear(); iRecs.resize(0);
  if (isII) {
    for (int j=0; j<(int)partsSav[iSys].size(); ++j) {
      if (partsSav[iSys][j].isFinal()) {
        recoiler.push_back(partsSav[iSys][j].p());
        iRecs.push_back(indexSav[iSys][j]);
      }
    }
  }
    
  //------------------------------------------------------------------------
  // FORCED SPLITTINGS
  // Check if we have to force a splitting (to get rid of heavy quarks)
  bool forceSplitting = false;
  if (winnerPtr->forceSplittingSav) {
    forceSplitting = true;
    if (verbose >= 6)
      printOut("SpaceAntShower::branch", "forcing a splitting at "
        "q = " + num2str(qNew));
    // If so, can we force the splitting at the scale we'd like to?
    double e1    = winnerPtr->e1();
    double q2max = winnerPtr->trialGenPtrSav[iTrial]->getQ2max(winnerPtr->s(), e1,
      is1A ? eAused : eBused);
    if (q2max < q2new) {
      q2new = q2max;
      qNew  = sqrt(q2new);
      // Safety * 0.99
      winnerPtr->scaleSav[iTrial] = 0.99*qNew;
      if (verbose >= 6)
        printOut("SpaceAntShower::branch", "adjusted scale to q = " + num2str(qNew));
    }
  }
    
  //------------------------------------------------------------------------
  // GENERATE FULL TRIAL KINEMATICS (AND REJECT IF OUTSIDE PHASE SPACE)
  if (!generateKinematics(winnerPtr, recoiler)) {
    // Mark this trial as "used", will need to generate a new one.
    winnerPtr->renewTrial(iTrial);
    return false;
  }
  // If we have a force splitting update scale information
  // might have change to ensure kinematics can be generated
  if (forceSplitting) {
    nForceSplit++;
    qNew  = winnerPtr->getTrialScale(iTrial);
    q2new = pow2(qNew);
  }

  //------------------------------------------------------------------------
  // ASSIGN COLOUR FLOW
  // NOTE: Using lastColTag here, if branching accepted we tell Pythia
  bool usedColTag = assignColourFlow(event, winnerPtr);
    
  //------------------------------------------------------------------------
  // UPDATE LIST OF PARTONS
  vector<Particle> partsNow = partsSav[iSys];
  vector<int> antPos; 
  antPos.resize(3);
  // Replace the old ones ...
  for (int i = 0; i < (int)indexSav[iSys].size(); ++i) {
    if (indexSav[iSys][i] == winnerPtr->iOld1) {
      partsNow[i] = winnerPtr->new1;
      antPos[0]   = i;
    }
    else if (indexSav[iSys][i] == winnerPtr->iOld2) {
      partsNow[i] = winnerPtr->new3;
      antPos[1]   = i;
    }
    else { // Check for recoiler
      for (int k=0; k<(int)iRecs.size(); k++)
        if (iRecs[k] == indexSav[iSys][i]) partsNow[i].p(recoiler[k]);
    }
  }
  // ... and add the new ones.
  partsNow.push_back(winnerPtr->new2); 
  antPos[2] = (int)partsNow.size()-1;
    
  //------------------------------------------------------------------------
  // PHASE SPACE CLOSED FOR GETTING RID OF HEAVY QUARKS?
  if (!forceSplitting) if (massCorrectionLevel > 0) if (!checkHeavyQuarkPhaseSpace(partsNow,iSys)) {
        if (verbose >= 6)
          printOut("SpaceAntShower::branch", "branching rejected because phase "
            "space after branching does not allow forced splittings");
        // Mark this trial as "used", will need to generate a new one.
        winnerPtr->renewTrial(iTrial);
        nClosePSforHQ++;
        return false;
      }
    
  // Sanity Check: Cutoff
  double qCutoffNow = (isII ? cutoffScaleII : cutoffScaleIF);
  // Check only if we don't force a splitting
  if (!forceSplitting) if (sqrt(q2new) < qCutoffNow) {
      if (iAntPhys==iQXsplitIF && winnerPtr->old1.idAbs()>nFlavZeroMass) {}
      else if (iAntPhys==iQXsplitII && !isSwapped && winnerPtr->old1.idAbs()>nFlavZeroMass) {}
      else if (iAntPhys==iQXsplitII &&  isSwapped && winnerPtr->old2.idAbs()>nFlavZeroMass) {}
      else {
        // Mark this trial as "used", will need to generate a new one.
        winnerPtr->renewTrial(iTrial);
        nTrialsCutQ++;
        return false;
      }
    }
  
  //------------------------------------------------------------------------
  // VETO STEP: DECIDE WETHER TO ACCEPT OR REJECT BRANCHING
  // Skip for forced splitting
  if (!forceSplitting) {
    if (!acceptTrial(event, winnerPtr, iSys, partsNow, antPos)) {
      // Mark this trial as "used", will need to generate a new one.
      winnerPtr->renewTrial(iTrial);
      return false;
    }
  }
  
  //------------------------------------------------------------------------
  // PUT NEW PARTICLES INTO EVENT RECORD
  // Store a copy of event, to be used if branching vetoed by userHooks
  Event evtOld    = event;
  int evtSizeOld  = evtOld.size();
  int iOld1       = winnerPtr->iOld1;
  int iOld2       = winnerPtr->iOld2;
  winnerPtr->new1.scale(qNew);
  winnerPtr->new2.scale(qNew);
  winnerPtr->new3.scale(qNew);
  int iNew1       = event.append(winnerPtr->new1);
  int iNew3       = event.append(winnerPtr->new3);
  int iNew2       = event.append(winnerPtr->new2);
  // Update the recoilers for II branchings
  vector< pair<int,int> > iRecNew; iRecNew.clear(); iRecNew.resize(0);
  if (isII) {
    for (int j=0; j<event.size(); ++j) if (event[j].isFinal()) {
        for (int k=0; k<(int)iRecs.size(); k++) if (iRecs[k] == j) {
            int inew = event.copy(j,44); // Copy the recoiler and 
            event[inew].p(recoiler[k]);  // change the momentum
            iRecNew.push_back(make_pair(iRecs[k],inew));
          }
      }
  }
  // Update event pointers if necessary
  event.restorePtrs();
  // Check if we went from polarized to unpolarized state
  // If so, depolarize parton state
  // (A more complete alternative here would be to create depolarized 
  // copies of all partons and then update everything, but deemed
  // unnecessary for now.)
  if ( event[iOld1].pol() != 9 && event[iOld2].pol() != 9 && 
    (event[iNew1].pol() == 9 || event[iNew2].pol() == 9 ||
      event[iNew3].pol() == 9)) {
    if (verbose >= 5) 
      printOut("SpaceAntShower::branch","Depolarizing parton state");
    // Depolarize parton state (except the branching mothers, 
    // which will be replaced by unpolarized daughters) 
    int sizeSystem = partonSystemsPtr->sizeOut(iSysSel);
    for (int i = 0; i < sizeSystem; ++i) {    
      int i1 = partonSystemsPtr->getOut( iSysSel, i);
      // Skip if not present in final state
      if ( i1 <= 0 || !event[i1].isFinal()) continue;
      // Skip if mother parton to be replaced by daughter
      if ( i1 == winnerPtr->iOld1 || i1 == winnerPtr->iOld2) continue;
      // Else depolarize
      if ( event[i1].pol() != 9 ) event[i1].pol(9);
    }
    // Also make sure all daughters are depolarized as well
    winnerPtr->new1.pol(9);
    winnerPtr->new2.pol(9);
    winnerPtr->new3.pol(9);
  }
    
  //------------------------------------------------------------------------
  // SET MOTHERS AND DAUGHTERS
  // Mark original dipole partons as branched
  event[iOld1].statusNeg();
  event[iOld2].statusNeg();
  if (isII) { // AB    ajb
    // Set mothers/daughters
    // Beam -> iNew1=a -> iOld1=A
    //                 |
    //                  iNew2=j
    //                 |
    // Beam -> iNew3=b -> iOld2=B
    // New initial partons inherit mothers (beam)
    event[iNew1].mothers(event[iOld1].mother1(), event[iOld1].mother2());
    event[iNew3].mothers(event[iOld2].mother1(), event[iOld2].mother2());
    // Gluon emission
    if (event[iNew2].id()==21) {
      event[iNew1].daughters(iNew2,iOld1); // iNew1 a inherits mothers of A, daughters are A and j
      event[iNew3].daughters(iNew2,iOld2); // iNew3 b inherits mothers of B, daughters are B and j
      event[iNew2].mothers(iNew1,iNew3);   // iNew2 j gets a and b as mothers, no daughters
    }
    // Gluon splitting or conversion in the initial state: side A
    else if (!isSwapped) {
      event[iNew1].daughters(iNew2,iOld1); // iNew1 a inherits mothers of A, daughters are A and j
      event[iNew3].daughters(iOld2,0);     // iNew3 b inherits mothers of B, daughter is B
      event[iNew2].mothers(iNew1,0);       // iNew2 j gets a as mother, no daughters
    }
    // Gluon splitting or conversion in the initial state: side B
    else {
      event[iNew1].daughters(iOld1,0);     // iNew1 a inherits mothers of A, daughter is A
      event[iNew3].daughters(iNew2,iOld2); // iNew3 b inherits mothers of B, daughters are B and j
      event[iNew2].mothers(iNew3,0);       // iNew2 j gets b as mother, no daughters
    }
    event[iOld1].mothers(iNew1,0);         // iOld1 A keeps its daughters, gets a as mother
    event[iOld2].mothers(iNew3,0);         // iOld2 B keeps its daughters, gets b as mother
    event[iNew2].daughters(0,0);           // iNew2 j has no daughters
    // Put a and b as daughters of the beam for hard process
    if (hardSys[iSys]) {
      bool founda = false;
      bool foundb = false;
      for (int i=0; i<(int)event.size(); i++) {
        if (!founda) { if (event[i].daughter1() == iOld1) {
            event[i].daughters(iNew1, 0);
            founda = true;
          } }
        if (!foundb) { if (event[i].daughter1() == iOld2) {
            event[i].daughters(iNew3, 0);
            foundb = true;
          } }
        if (founda && foundb) break;
      }
    }
  }
  else { // AK    ajk
    // Set mothers/daughters
    //                   iNew2=j
    //                  |     | 
    //                 |      iOld2=K -> iNew3=k
    // Beam -> iNew1=a -> iOld1=A
    // New initial parton inherits mothers (beam)
    event[iNew1].mothers(event[iOld1].mother1(), event[iOld1].mother2());
    // Gluon emission
    if (event[iNew2].id()==21) {
      event[iNew1].daughters(iNew2,iOld1); // iNew1 a inherits mothers of A, daughters are A and j
      event[iOld2].daughters(iNew2,iNew3); // iOld2 K gets k and j as daughters, keeps its mothers
      event[iNew3].mothers(iOld2,0);       // iNew3 k gets K as mother, no daughters
      event[iNew2].mothers(iOld2,iNew1);   // iNew2 j gets a and K as mothers, no daughters
    }
    // Gluon splitting or conversion in the initial state
    else if (iAntPhys==iQXsplitIF || iAntPhys==iGXconvIF) {
      event[iNew1].daughters(iNew2,iOld1); // iNew1 a inherits mothers of A, daughters are A and j
      event[iOld2].daughters(iNew3,0);     // iOld2 K gets k as daughter, keeps its mothers
      event[iNew3].mothers(iOld2,0);       // iNew3 k gets K as mother, no daughters
      event[iNew2].mothers(iNew1,0);       // iNew2 j gets a as mother, no daughters
    }
    // Gluon splitting in the final state
    else {
      event[iNew1].daughters(iOld1,0);     // iNew1 a inherits mothers of A, daughteris A
      event[iOld2].daughters(iNew2,iNew3); // iOld2 K gets k and j as daughters, keeps its mothers
      event[iNew3].mothers(iOld2,0);       // iNew3 k gets K as mother, no daughters
      event[iNew2].mothers(iOld2,0);       // iNew2 j gets K as mother, no daughters
    }
    event[iOld1].mothers(iNew1,0);         // iOld1 A keeps its daughters, gets a as mother
    event[iNew3].daughters(0,0);           // iNew3 5 has no daughters
    event[iNew2].daughters(0,0);           // iNew2 j has no daughters
    // Put a as daughter of the beam for hard process
    if (hardSys[iSys]) {
      for (int i=0; i<(int)event.size(); i++) {
        if (event[i].daughter1() == iOld1) {
          event[i].daughters(iNew1, 0);
          break;
        }
      }
    }
  }
    
  //------------------------------------------------------------------------
  // VETO BY USERHOOKS
  // Possibility to allow user veto of emission step.
  if (canVetoEmission)
    if (userHooksPtr->doVetoISREmission(evtSizeOld, event, iSys)) {
      // Restore old event
      event = evtOld;
      return false;
    }

  //------------------------------------------------------------------------
  // EVERYTHING ACCEPTED

  //------------------------------------------------------------------------
  // UPDATE LIST OF SYSTEMS
  partonSystemsPtr->replace(iSys, iOld1, iNew1);
  partonSystemsPtr->addOut(iSys, iNew2);
  partonSystemsPtr->replace(iSys, iOld2, iNew3);
  // Recoilers and initial guys
  if (isII) {
    for (int k=0; k<(int)iRecNew.size(); k++)
      partonSystemsPtr->replace(iSys, iRecNew[k].first, iRecNew[k].second);
    partonSystemsPtr->setInA(iSys, iNew1);
    partonSystemsPtr->setInB(iSys, iNew3);
  }
  else if (is1A) partonSystemsPtr->setInA(iSys, iNew1);
  else partonSystemsPtr->setInB(iSys, iNew1);
  double shat = (event[partonSystemsPtr->getInA(iSys)].p() + 
    event[partonSystemsPtr->getInB(iSys)].p()).m2Calc();
  partonSystemsPtr->setSHat(iSys, shat);
    
  //------------------------------------------------------------------------
  // UPDATE BEAM PARTICLES
  bool isValN1 = false;
  bool isValN3 = false;
  if (isII) { // Update both sides
    // side A
    BeamParticle& beam1 = *beamAPtr;
    beam1[iSys].update( iNew1, event[iNew1].id(), event[iNew1].e()/eBeamA );
    // Redo choice of companion kind whenever new flavour.
    if (event[iOld1].id() != event[iNew1].id()) {
      double PDFscale = q2new;
      beam1.xfISR( iSys, event[iNew1].id(), event[iNew1].e()/eBeamA, PDFscale );
      beam1.pickValSeaComp();
    }
    isValN1 = beam1[iSys].isValence();
    // side B
    BeamParticle& beam2 = *beamBPtr;
    beam2[iSys].update( iNew3, event[iNew3].id(), event[iNew3].e()/eBeamB );
    // Redo choice of companion kind whenever new flavour.
    if (event[iOld2].id() != event[iNew3].id()) {
      double PDFscale = q2new;
      beam2.xfISR( iSys, event[iNew3].id(), event[iNew3].e()/eBeamB, PDFscale );
      beam2.pickValSeaComp();
    }
    isValN3 = beam2[iSys].isValence();
  }
  else { // Update only one side
    BeamParticle& beam = (is1A ? *beamAPtr : *beamBPtr);
    beam[iSys].update( iNew1, event[iNew1].id(), event[iNew1].e()/(is1A ? eBeamA : eBeamB) );
    // Redo choice of companion kind whenever new flavour.
    if (event[iOld1].id() != event[iNew1].id()) {
      double PDFscale = q2new;
      beam.xfISR( iSys, event[iNew1].id(), event[iNew1].e()/(is1A ? eBeamA : eBeamB), PDFscale );
      beam.pickValSeaComp();
    }
    isValN1 = beam[iSys].isValence();
  }
    
  //------------------------------------------------------------------------
  // UPDATE ANTENNAE DUE TO RECOIL
  if (isII) {
    for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
      BranchelementalISR* antPtr = &branchelementals[iSys][iAnt];
      int i2  = antPtr->iOld2;
      int iLa = antPtr->iL;
      int iRa = antPtr->iR;
      // Check relevant partons for recoilers
      for (int k=0; k<(int)iRecNew.size(); k++) {
        // i1 is always initial
        if (i2  == iRecNew[k].first) antPtr->iOld2 = iRecNew[k].second;
        if (iLa == iRecNew[k].first) antPtr->iL    = iRecNew[k].second;
        if (iRa == iRecNew[k].first) antPtr->iR    = iRecNew[k].second;
      }
    }
  }
    
  // Update number of gluons and quark pairs
  if (iAntPhys == iGXconvII || iAntPhys == iGXconvIF || iAntPhys == iXGsplitIF) {
    nG[iSys] -= 1;
    ++nQQ[iSys];
  } else ++nG[iSys];
    
  //------------------------------------------------------------------------
  // UPDATE LIST OF PARTONS AND PARTON NUMBERS
  indexSav[iSys].clear(); indexSav[iSys].resize(0);
  partsSav[iSys].clear(); partsSav[iSys].resize(0);
  int sizeSystem = partonSystemsPtr->sizeAll(iSys);
  for (int i = 0; i < sizeSystem; ++i) {    
    int i1 = partonSystemsPtr->getAll( iSys, i);
    if ( i1 <= 0 ) continue;
    indexSav[iSys].push_back( i1 );
    partsSav[iSys].push_back( event[i1] );
    if (!event[i1].isFinal()) {
      if (event[i1].pz() > 0.0) initialA[iSys] = event[i1];
      else initialB[iSys] = event[i1];
    }
  }
  eAused = 0.0;
  eBused = 0.0;
  for (int i = 0; i < (int)initialA.size(); i++) {
    eAused += initialA[i].e();
    eBused += initialB[i].e();
  }
  if (verbose >= 6) {
    printOut("SpaceAntShower::branch", "updated system, now we have "
      "the following particles: nr id col acol p");
    for (int i=0; i<(int)partsSav[iSys].size(); i++) {
      cout << "  " << setw(3) << indexSav[iSys][i];
      cout << " " << setw(2) << partsSav[iSys][i].id();
      cout << " " << setw(3) << partsSav[iSys][i].col();
      cout << " " << setw(3) << partsSav[iSys][i].acol();
      cout << " " << partsSav[iSys][i].p();
    }
  }

  // Communicate changed lists to FSR shower
  FSRshowerPtr->partsSav = partsSav;
  FSRshowerPtr->indexSav = indexSav;
  
  // Verbose output
  if (verbose >= 5)
    printOut("SpaceAntShower::branch",
      "=== Branching Accepted. Updating dipole-antenna(e)");
    
  //************************************************************************
  // Updates of the branched antennae
  //************************************************************************
    
  // Store old neighbours
  int iL = winnerPtr->iL;
  int iR = winnerPtr->iR;
    
  //------------------------------------------------------------------------
  // 1) Gluon Emission
  if (winnerPtr->new2.id() == 21) {
    // II -> we created two IF antennae
    // IF -> we created an IF and FF (don't care) antenna
      
    // Old one
    winnerPtr->reset(event,iNew1,iNew2,isValN1,false);
    winnerPtr->system = iSys; // Save system
    resetTrialGenerators(winnerPtr); // Update trial generators
    // Update neighbours
    winnerPtr->iL = iL;
    winnerPtr->iR = iNew3;
    // Update colour, old2 is gluon, so both col and acol != 0
    winnerPtr->col = ( (winnerPtr->old1.col() == winnerPtr->old2.col()) ?
      winnerPtr->old2.col() : winnerPtr->old2.acol() );
      
    // II -> add the other IF antenna
    if (isII) {
      BranchelementalISR newTrial(event,iNew3,iNew2,isValN3,false);
      newTrial.system = iSys; // Save system
      resetTrialGenerators(&newTrial); // Update trial generators
      // Update neighbours
      newTrial.iL = iR;
      newTrial.iR = iNew1;
      // Update colour, old2 is gluon, so both col and acol != 0
      newTrial.col = ( (newTrial.old1.col() == newTrial.old2.col()) ?
        newTrial.old2.col() : newTrial.old2.acol() );
      // Save branchelemental
      branchelementals[iSys].push_back(newTrial);
    }
  }
    
  //------------------------------------------------------------------------
  // 2) Gluon splitting in the initial state:
  else if (iAntPhys == iQXsplitII || iAntPhys == iQXsplitIF) {
    // Keep the old antenna and add an IF antenna
      
    // Old one
    winnerPtr->reset(event,iNew1,iNew3,isValN1,isValN3);
    winnerPtr->system = iSys; // Save system
    resetTrialGenerators(winnerPtr); // Update trial generators
    // Update neighbours
    if (isSwapped) { // Left side / side A didn't change
      winnerPtr->iL = iL;
      winnerPtr->iR = iNew2;
    } else { // Right side / side B didn't change
      winnerPtr->iL = iNew2;
      winnerPtr->iR = iR;
    }
    // Update colour, one is gluon, so both col and acol != 0
    if (winnerPtr->isII())
      winnerPtr->col = ( (winnerPtr->old1.col() == winnerPtr->old2.acol()) ?
        winnerPtr->old2.acol() : winnerPtr->old2.col() );
    else
      winnerPtr->col = ( (winnerPtr->old1.col() == winnerPtr->old2.col()) ?
        winnerPtr->old2.col() : winnerPtr->old2.acol() );
      
    // Add the other IF antenna
    int iSplitGluon = (isSwapped ? iNew3 : iNew1);
    BranchelementalISR newTrial(event,iSplitGluon,iNew2,false,false);
    newTrial.system = iSys; // Save system
    resetTrialGenerators(&newTrial); // Update trial generators
    // Update neighbours
    newTrial.iL = (isSwapped ? iNew1 : iNew3); 
    newTrial.iR = 0; // 2 is quark
    // Update colour, old1 is gluon, so both col and acol != 0
    newTrial.col = ( (newTrial.old1.col() == newTrial.old2.col()) ?
      newTrial.old2.col() : newTrial.old2.acol() );
    // Save branchelemental
    branchelementals[iSys].push_back(newTrial);
  }
      
  //------------------------------------------------------------------------
  // 3) Gluon convsersion in the initial state:
  else if (iAntPhys == iGXconvII || iAntPhys == iGXconvIF) {
    // Update branched antenna, check IS or FS quark carry ant colour
    int iFSQ     = iNew2;
    int colFSQ   = ( event[iFSQ].id() > 0 ? event[iFSQ].col()
      : event[iFSQ].acol() );
    int iISQ     = (isSwapped ? iNew3 : iNew1);
    int colISQ   = ( event[iISQ].id() > 0 ? event[iISQ].col()
      : event[iISQ].acol() );
    int iQLeft   = 0;
    int colQLeft = 0;
    bool isValQL = false;
    if (colFSQ == winnerPtr->col) {
      // In case this is FF ant: Kick out later
      int iPartner = (isSwapped ? iNew1 : iNew3);
      bool isPval  = (isSwapped ? isValN1 : isValN3);
      winnerPtr->reset(event,iPartner,iFSQ,isPval,false);
      winnerPtr->iL  = iL; // Take org
      winnerPtr->iR  = 0;  // 0 since quark
      winnerPtr->col = colFSQ;
      // IS quark is left
      iQLeft   = iISQ;
      colQLeft = colISQ;
      isValQL  = (isSwapped ? isValN3 : isValN1);
    } else if (colISQ == winnerPtr->col) {
      // Is II or IF ant
      winnerPtr->reset(event,iNew1,iNew3,isValN1,isValN3);
      if (isSwapped) { // iNew3 is initial quark
        winnerPtr->iL  = iL; // Take org
        winnerPtr->iR  = 0;  // 0 since quark
      } else { // iNew1 is initial quark
        winnerPtr->iL  = 0;  // 0 since quark
        winnerPtr->iR  = iR; // Take org
      }
      winnerPtr->col = colISQ;
      // FS quark is left
      iQLeft   = iFSQ;
      colQLeft = colFSQ;
    }
    winnerPtr->system = iSys; // Save system
    resetTrialGenerators(winnerPtr); // Update trial generators
      
    // Find other antenna, where converted gluon took part in,
    // replace gluon with left quark
    int iConvGluon = (isSwapped ? iOld2 : iOld1);
      
    // Can only find one ant with old gluon, as winner already updated
    for (int iAnt=0; iAnt<(int)branchelementals[iSys].size(); iAnt++) {   
      BranchelementalISR* antPtr = &branchelementals[iSys][iAnt];
      int iLkeep = antPtr->iL;
      int iRkeep = antPtr->iR;
      if (antPtr->iOld1 == iConvGluon) {
        antPtr->reset(event,antPtr->iOld2,iQLeft,antPtr->isVal2,isValQL);
        if (antPtr->iOld2 == iQLeft) {
          antPtr->iL  = iRkeep; // Take org
          antPtr->iR  = 0;      // 0 since quark
        } else {
          antPtr->iL  = 0;      // 0 since quark
          antPtr->iR  = iRkeep; // Take org
        }
        antPtr->col = colQLeft;
        resetTrialGenerators(antPtr); // Update trial generators
      } else if (antPtr->iOld2 == iConvGluon) {
        // New antenna is IF
        antPtr->reset(event,antPtr->iOld1,iQLeft,antPtr->isVal1,isValQL);
        if (antPtr->iOld2 == iQLeft) {
          antPtr->iL  = iLkeep; // Take org
          antPtr->iR  = 0;      // 0 since quark
        } else {
          antPtr->iL  = 0;      // 0 since quark
          antPtr->iR  = iLkeep; // Take org
        }
        antPtr->col = colQLeft;
        resetTrialGenerators(antPtr); // Update trial generators
      }
      antPtr->system = iSys; // Save system
    }
  }

  //------------------------------------------------------------------------
  // 4) Gluon splitting in the final state:
  else if (iAntPhys == iXGsplitIF) {
    // Keep the old antenna (iNew2 as emission and iNew1 as initial partner)
    // which is IF, no new one created
    winnerPtr->reset(event,iNew1,iNew2,isValN1,false);
    winnerPtr->system = iSys; // Save system
    resetTrialGenerators(winnerPtr); // Update trial generators
    // Update neighbours
    winnerPtr->iL = iL; // initial stays
    winnerPtr->iR = 0;  // final has no one
    // Update colour, old2 is quark so if col2 !=0 that's antenna colour
    winnerPtr->col = ( (winnerPtr->old2.col() != 0) ? 
      winnerPtr->old2.col() : winnerPtr->old2.acol() );
    // And check the other antenna iOld2 was involved: has to be IF
    for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
      BranchelementalISR* antPtr = &branchelementals[iSys][iAnt];
      if (antPtr->iOld2 == iOld2) {
        antPtr->iOld2 = iNew3;
        antPtr->old2  = event[iNew3];
        antPtr->iR = 0; // No neighbour on final side because it's a quark
        // No changes on initial side, since no branching in this antenna
        resetTrialGenerators(antPtr); // Update trial generators
      }
    }
  }
    
  //************************************************************************
  // Updates of the other antennae
  //************************************************************************
    
  //------------------------------------------------------------------------
  // Updates
  for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
    BranchelementalISR* antPtr = &branchelementals[iSys][iAnt];
    antPtr->system = iSys;  // Save system
      
    int i1  = antPtr->iOld1;
    int i2  = antPtr->iOld2;
    int iLa = antPtr->iL;
    int iRa = antPtr->iR;
      
    // Update neighbours
    if (iLa == iOld1) antPtr->iL = iNew1;
    else if (iLa == iOld2) antPtr->iL = iNew3;
    if (iRa == iOld1) antPtr->iR = iNew1;
    else if (iRa == iOld2) antPtr->iR = iNew3;
      
    // Update particles (updates with flavour changes are already done before)
    // so we don't need to update the trial generators here
    if ( (i1 == iOld1) || (i1 == iOld2) || (i2 == iOld1) || (i2 == iOld2) ) {
      antPtr->renewTrial();
      if (i1 == iOld1) { 
        antPtr->iOld1  = iNew1; 
        antPtr->isVal1 = isValN1;
      } else if (i1 == iOld2) { 
        antPtr->iOld1  = iNew3; 
        antPtr->isVal1 = isValN3;
      }
      if (i2 == iOld1) { 
        antPtr->iOld2  = iNew1; 
        antPtr->isVal2 = isValN1;
      } else if (i2 == iOld2) {
        antPtr->iOld2  = iNew3; 
        antPtr->isVal2 = isValN3;
      }
    }
      
    // Update
    i1  = antPtr->iOld1;
    i2  = antPtr->iOld2;
    iLa = antPtr->iL;
    iRa = antPtr->iR;
      
    // Check relevant partons for recoilers
    for (int k=0; k<(int)iRecNew.size(); k++) {
      // i1 is always initial
      if (i2 == iRecNew[k].first) {
        antPtr->iOld2 = iRecNew[k].second;
        antPtr->renewTrial();
      }
      if (iLa == iRecNew[k].first) antPtr->iL = iRecNew[k].second;
      if (iRa == iRecNew[k].first) antPtr->iR = iRecNew[k].second;
    }
    
    // Update the particles (eg if a recoil changed a parent momentum)
    i1 = antPtr->iOld1;
    i2 = antPtr->iOld2;
    antPtr->old1 = event[i1];
    antPtr->old2 = event[i2];
      
    antPtr->mCalc();

    // Reset rescue counter
    antPtr->resetRescue();
  }
      
  //------------------------------------------------------------------------
  // Sanity check: Kick out any FF antenna we might have created
  for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
    BranchelementalISR* antPtr = &branchelementals[iSys][iAnt];
    if (antPtr->old1.isFinal() && antPtr->old2.isFinal())
      branchelementals[iSys].erase(branchelementals[iSys].begin()+iAnt);
  }
    
  //------------------------------------------------------------------------
  // Renew trials in all other systems since pdfs changed
  for (int iSystNow = 0; iSystNow < (int)branchelementals.size(); iSystNow++) 
    if (iSystNow != iSys) {
      for (int iAnt = 0; iAnt < (int)branchelementals[iSystNow].size(); ++iAnt) {
        // Reset rescue counter
        branchelementals[iSystNow][iAnt].resetRescue();
        if ( isII || (is1A && branchelementals[iSystNow][iAnt].is1A())
          || (!is1A && !branchelementals[iSystNow][iAnt].is1A()) ) {
          branchelementals[iSystNow][iAnt].renewTrial();
        }
      }
    }
  
  if (verbose >= 5) {
    event.list();
    list();
  }
    
  // Count the number of branchings in the system
  nBranch[iSys]++;
  nBranchISR[iSys]++;
    
  // Let Pythia know how many color tags we used
  if (usedColTag) {
    event.nextColTag();
    if (event[iNew2].id() == 21) {
      int lastTag = event.lastColTag();
      int colMax  = max(event[iNew2].col(),event[iNew2].acol());
      while (colMax > lastTag) lastTag = event.nextColTag();
    }
  }
    
  // Check the event after each branching
  vinComPtr->showerChecks(event, true);
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::branch","finished --------------");

  // Statistics for first 10 branchings in first 4 systems
  if (iSys < 4 && nBranchISR[iSys] < 11) {
    qBranch[iSys][nBranchISR[iSys]]=qNew;
    pTphys[iSys][nBranchISR[iSys]]=event[iNew2].pT();
  }

  // Done.
  return true;
    
}
  
//**********************************************************************
  
// END OF INHERITED FUNCTIONS FOR INTERFACE TO PYTHIA8
// BEGIN VINCIA INTERNAL FUNCTIONS
  
//**********************************************************************
  
//*********
  
// Update antennae if FSR changed system in prepare
  
void SpaceAntShower::updateAfterFSR(int iSys, Event& event, map<int,int> iChanged) {
    
  // Don't need to do anything
  if ((int)branchelementals.size() <= iSys ) return;
  if (branchelementals[iSys].size() <= 0) return;
    
  // Update the particles in my list
  for (int i=0; i<(int)indexSav[iSys].size(); i++) {
    int iOld = indexSav[iSys][i];
    indexSav[iSys][i] = iChanged[iOld];
    partsSav[iSys][i] = event[iChanged[iOld]];
    if (!partsSav[iSys][i].isFinal()) {
      // Update the beam pointers
      BeamParticle& beamNow = (partsSav[iSys][i].pz() > 0.0 ? *beamAPtr : *beamBPtr);
      double eBeamNow       = (partsSav[iSys][i].pz() > 0.0 ? eBeamA : eBeamB);
      beamNow[iSys].update( iChanged[iOld], partsSav[iSys][i].id(), 
        partsSav[iSys][i].e()/eBeamNow );
    }
  }
    
  // Loop over the antennae to reset antenna partons and neighbours
  for (int i=0; i<(int)branchelementals[iSys].size(); i++) {
    BranchelementalISR* antPtr = &branchelementals[iSys][i];
    int iOld1 = antPtr->iOld1;
    int iOld2 = antPtr->iOld2;
    antPtr->iOld1 = iChanged[iOld1];
    antPtr->iOld2 = iChanged[iOld2];
    antPtr->old1  = event[iChanged[iOld1]];
    antPtr->old2  = event[iChanged[iOld2]];
    // Recalculate the mass
    antPtr->mCalc();
    // Trialgenerators are still the same
    // Neighbours
    if (antPtr->iL != 0) antPtr->iL = iChanged[antPtr->iL];
    if (antPtr->iR != 0) antPtr->iR = iChanged[antPtr->iR];
    // Update daughter partons
    iOld1 = antPtr->iOld1;
    iOld2 = antPtr->iOld2;
    antPtr->new1 = Particle(0,-41,iOld1,iOld2,0,0,0,0,0.);
    antPtr->new2 = Particle(0,43,iOld1,iOld2,0,0,0,0,0.);
    antPtr->new3 = Particle(0,antPtr->isII()?-41:44,iOld1,iOld2,0,0,0,0,0.);
    antPtr->new1.setEvtPtr(&event);
    antPtr->new2.setEvtPtr(&event);
    antPtr->new3.setEvtPtr(&event);
    // Update antenna colour (can't have two II, because FSR would not
    // touch this system
    if (antPtr->old1.idAbs() < 6) {
      if (antPtr->old1.id() > 0) antPtr->col = antPtr->old1.col();
      else antPtr->col = antPtr->old1.acol();
    } else {
      int col = antPtr->old1.col();
      if (col != ( antPtr->isII() ? antPtr->old2.acol() : antPtr->old2.col() ))
        col = antPtr->old1.acol();
      antPtr->col = col;
    }
  }
    
}
  
//*********
  
// Add trial functions to a Branchelemental
  
void SpaceAntShower::resetTrialGenerators(BranchelementalISR* trial) {
    
  // Reset
  trial->clearTrialGenerators();
    
  // Parents and are we II?
  int id1            = trial->old1.idAbs(); // If II always p+ guy. if IF always I guy
  int id2            = trial->old2.idAbs(); // If II always p- guy, if IF always F guy
  bool isOctetOnium2 = ( (id2>6 && id2!=21) ? true : false );
  bool isVal1        = trial->isVal1;
  bool isVal2        = trial->isVal2;
  bool isII          = trial->isII();
  bool is1A          = trial->is1A();
    
  int iAntPhys = -1;

  if (isII) {
    // QQbar
    if ( id1 < 6 && id2 < 6 ) {
      iAntPhys  = iQQemitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, false, &trialIISoft);
      iAntPhys  = iQXsplitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        if (!isVal1) trial->addTrialGenerator(iAntPhys, false, &trialIISplitA);
        if (!isVal2) trial->addTrialGenerator(iAntPhys, true, &trialIISplitB);
      }
    }
    // GG
    else if ( id1 == 21 && id2 == 21 ) {
      iAntPhys  = iGGemitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, false, &trialIISoft);
        trial->addTrialGenerator(iAntPhys, false, &trialIIGCollA);
        trial->addTrialGenerator(iAntPhys, true, &trialIIGCollB);
      }
      iAntPhys  = iGXconvII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, false, &trialIIConvA);
        trial->addTrialGenerator(iAntPhys, true, &trialIIConvB);
      }
    }
    // QG
    else if ( id1 < 6 && id2 == 21 ) {
      iAntPhys  = iGQemitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, true, &trialIISoft);
        trial->addTrialGenerator(iAntPhys, true, &trialIIGCollB);
      }
      iAntPhys = iGXconvII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, true, &trialIIConvB);
      iAntPhys = iQXsplitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        if (!isVal1) trial->addTrialGenerator(iAntPhys, false, &trialIISplitA);
    }
    // GQ
    else if ( id1 == 21 && id2 < 6 ) {
      iAntPhys = iGQemitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, false, &trialIISoft);
        trial->addTrialGenerator(iAntPhys, false, &trialIIGCollA);
      }
      iAntPhys = iGXconvII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, false, &trialIIConvA);
      iAntPhys = iQXsplitII;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        if (!isVal2) trial->addTrialGenerator(iAntPhys, true, &trialIISplitB);
    }
  }
    
  else {
    // QQ
    if ( id1 < 6 && id2 < 6 ) {
      iAntPhys = iQQemitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSoft);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFQCollA);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFQCollK);
      }
      iAntPhys = iQXsplitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        if (!isVal1) trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSplitA);
    }
    // GG
    else if ( id1 == 21 && id2 == 21 ) {
      iAntPhys = iGGemitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSoft);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollA1);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollA2);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollK);
      }
      iAntPhys = iXGsplitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSplitK);
      iAntPhys = iGXconvIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFConvA);
    }
    // GQ
    else if ( id1 == 21 && id2 < 6 ) {
      iAntPhys = iGQemitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSoft);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollA1);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollA2);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFQCollK);
      }
      iAntPhys = iGXconvIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFConvA);
    }
    // QG
    else if ( id1 < 6 && id2 == 21 ) {
      iAntPhys = iQGemitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.) {
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSoft);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFQCollA);
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFGCollK);
      }
      iAntPhys = iXGsplitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSplitK);
      iAntPhys = iQXsplitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        if (!isVal1) trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSplitA);
    }
    // GOctetOnium
    else if ( id1 == 21 && isOctetOnium2 ) {
      iAntPhys = iGXconvIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        trial->addTrialGenerator(iAntPhys, !is1A, &trialIFConvA);
    }
    // QOctetOnium
    else if ( id1 < 6 && isOctetOnium2 ) {
      iAntPhys = iQXsplitIF;
      if (getAnt(iAntPhys)->getChargeFactor() > 0.)
        if (!isVal1) trial->addTrialGenerator(iAntPhys, !is1A, &trialIFSplitA);
    }
  }
    
}
  
//*********
  
// Generate Kinematics
  
bool SpaceAntShower::generateKinematicsII(BranchelementalISR* trialPtr, vector<Vec4>& pRec) {
    
  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::generateKinematicsII","begin --------------");

  // Basic info about trial function and scale
  int iTrial     = indxWin;
  if (iTrial < 0) return false;
  double qNew    = trialPtr->getTrialScale(iTrial);
  double q2new   = pow2(qNew);
  int iAntPhys   = trialPtr->getPhysIndex(iTrial);
  bool isSwapped = trialPtr->getIsSwapped(iTrial);
    
  // Trial generator
  TrialGeneratorISR* trialGenPtr = trialPtr->trialGenPtrSav[iTrial];
  int trialGenStatInt            = trialGenPtr->statInt();
    
  // IDs
  int idA = trialPtr->old1.id();
  int idB = trialPtr->old2.id();
    
  // Are we forcing a splitting?
  bool forceSplitting = trialPtr->forceSplittingSav;
  if (forceSplitting) {
    // Set zMin and zMax accordingly
    trialPtr->zMinSav[iTrial] = 
      trialPtr->trialGenPtrSav[iTrial]->getZmin(q2new, trialPtr->s(), trialPtr->e1(),
        0.0);
    trialPtr->zMaxSav[iTrial] = 
      trialPtr->trialGenPtrSav[iTrial]->getZmax(q2new, trialPtr->s(), trialPtr->e1(),
        0.0);
  }
    
  //************************************************************************
  // Generate zeta variable, work out saj, sjb and check initial energies
  //************************************************************************
    
  double saj, sjb;
  bool pass = trialPtr->genTrialInvariants(saj, sjb, 0.0, iTrial);
  if (!pass) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII",
      "z outside physical range, returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }
    
  // Check that sab < shh
  double sAB = trialPtr->s();
  double sab = sAB + saj + sjb;
  if (sab > sbbSav) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII",
      "sab > shh = " + num2str(sbbSav) + ", returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }
    
  // Check the energies of the initial guys
  double eA = trialPtr->e1();
  double eB = trialPtr->e2();
  double ea = sqrt(eA/eB*(sAB+sjb)/(sAB+saj)*sab)/2.0;
  double eb = sqrt(eB/eA*(sAB+saj)/(sAB+sjb)*sab)/2.0;
  double eAusedNow = eAused - eA + ea;
  double eBusedNow = eBused - eB + eb;
  if ( (eAusedNow>0.98*eBeamA) || (eBusedNow>0.98*eBeamB) ) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII",
      "Energy of incoming guys exceed beam energy, returning.");
    if (forceSplitting) { // Lowering saj and sjb until success
      if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII",
        "Forced splitting, lowering saj and sjb now.");
      bool take     = false;
      double lowera = 0.01*saj;
      double lowerb = 0.01*sjb;
      do {
        saj = max(0.0,saj-lowera);
        sjb = max(0.0,sjb-lowerb);
        sab = sAB + saj + sjb;
        ea  = sqrt(eA/eB*(sAB+sjb)/(sAB+saj)*sab)/2.0;
        eb  = sqrt(eB/eA*(sAB+saj)/(sAB+sjb)*sab)/2.0;
        eAusedNow = eAused - eA + ea;
        eBusedNow = eBused - eB + eb;
        if ( (eAusedNow<0.98*eBeamA) && (eBusedNow<0.98*eBeamB) ) take = true;
      } while (!take && saj>0.0 && sjb>0.0);
      qNew  = trialGenPtr->getQ2(saj,sjb,sAB);
      q2new = pow2(qNew);
      trialPtr->scaleSav[iTrial] = qNew;
    }
    else {
      trialPtr->nHull++;
      nTrialsHull++;
      return false;
    }
  }
  else if ( (ea < eA) || (eb < eB) ) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII",
      "Energy of incoming guys decreased, returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }

  //************************************************************************
  // Set flavors and masses
  //************************************************************************
    
  // NOTE in gluon conversion the id of the quark that ends up as the initial
  // guy gets set in the trial generator when evaluating the pdf ratio, so we
  // have to do that here

  double PDFscale = q2new;
  double pdfRatio  = 1.0;
  if (trialGenStatInt < 3)
    // Always use PDFs in soft if combining trial generators for gluon emission
    pdfRatio = trialIISoft.physPDFratio(beamAPtr, beamBPtr, iSysWin, false,
      idA, idB, eA, eB, ea, eb, PDFscale, PDFscale);
  else pdfRatio = trialGenPtr->physPDFratio(beamAPtr, beamBPtr, iSysWin, false,
    idA, idB, eA, eB, ea, eb, PDFscale, PDFscale);
  trialPtr->addPDF(iTrial, pdfRatio); // save
  int gConvID = trialGenPtr->getGconvID();

  // Gluon emission: inherit parent flavors and add middle gluon 
  if (iAntPhys == iQQemitII || iAntPhys == iGQemitII || iAntPhys == iGGemitII) {
    trialPtr->new1.id(idA);
    trialPtr->new2.id(21);
    trialPtr->new3.id(idB);
    trialPtr->new1.m(trialPtr->old1.m());
    trialPtr->new2.m(0.0);
    trialPtr->new3.m(trialPtr->old2.m());
  }
    
  // Gluon splitting in the initial state:
  else if (iAntPhys == iQXsplitII) {
    if (!isSwapped) { // side A splitting
      trialPtr->new1.id(21);
      trialPtr->new2.id(-idA);
      trialPtr->new3.id(idB);
      trialPtr->new1.m(0.0);
      trialPtr->new2.m(trialPtr->old1.m());
      trialPtr->new3.m(trialPtr->old2.m());
    }
    else { // side B splitting
      trialPtr->new1.id(idA);
      trialPtr->new2.id(-idB);
      trialPtr->new3.id(21);
      trialPtr->new1.m(trialPtr->old1.m());
      trialPtr->new2.m(trialPtr->old2.m());
      trialPtr->new3.m(0.0);
    }
  }
    
  // Gluon conversion in the initial state:
  else if (iAntPhys == iGXconvII) {
    if (!isSwapped) { // side A conversion
      trialPtr->new1.id(gConvID);
      trialPtr->new2.id(gConvID);
      trialPtr->new3.id(idB);
      trialPtr->new1.m(0.0);
      trialPtr->new2.m(0.0);
      trialPtr->new3.m(trialPtr->old2.m());
    }
    else { // side B conversion
      trialPtr->new1.id(idA);
      trialPtr->new2.id(gConvID);
      trialPtr->new3.id(gConvID);
      trialPtr->new1.m(trialPtr->old1.m());
      trialPtr->new2.m(0.0);
      trialPtr->new3.m(0.0);
    }
  }
    
  //************************************************************************
  // Check local hadronization veto(s) : inside this Branchelemental
  //************************************************************************

  // Nothing to do here, II can only generate FF colour connection outside
  // the antenna
    
  //***********************************************************************
  // Generate full kinematics for this trial branching
  //***********************************************************************

  // Generate random (uniform) phi angle
  double phi = 2 * M_PI * rndmPtr->flat();
    
  // Generate branching kinematics, starting from dipole-antenna parents
  vector<Vec4> pOld, pNew;
  pOld.push_back(trialPtr->old1.p());
  pOld.push_back(trialPtr->old2.p());
  pass = vinCluPtr->map2to3masslessII(pNew, pRec, pOld, sAB, saj, sjb, sab, phi);
  if (!pass && !forceSplitting) {
    if (verbose >= 4) printOut("SpaceAntShower::generateKinematicsII",
      "Invariants do not match when calculating the momenta, returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }
  // Save momenta
  trialPtr->new1.p(pNew[0]);
  trialPtr->new2.p(pNew[1]);
  trialPtr->new3.p(pNew[2]);
  // Set default polarizations: unpolarized (may be assigned later)
  trialPtr->new1.pol(9); 
  trialPtr->new2.pol(9); 
  trialPtr->new3.pol(9);
    
  if (verbose >= 8) {
    printOut("SpaceAntShower::generateKinematicsII", "Printing pre-branching momenta");
    for (int i=0; i<2; i++) cout << "  " << pOld[i];
    printOut("SpaceAntShower::generateKinematicsII", "Printing post-branching momenta and recoiler");
    for (int i=0; i<3; i++) cout << "  " << pNew[i];
    for (int i=0; i<(int)pRec.size(); i++) cout << "  " << pRec[i];
  }
    
  // Note: colour flow is not assigned here, since this requires 
  // knowledge of which is the next global colour tag available in
  // the event.
    
  // Verbose output
  if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsII","finished --------------");
    
  return true;
    
}
  
bool SpaceAntShower::generateKinematicsIF(BranchelementalISR* trialPtr) {
    
  // Verbose output
  if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF","begin --------------");

  // Basic info about trial function and scale
  int iTrial   = indxWin;
  if (iTrial < 0) return false;
  double qNew  = trialPtr->getTrialScale(iTrial);
  double q2new = pow2(qNew);
  int iAntPhys = trialPtr->getPhysIndex(iTrial);
  bool is1A    = trialPtr->is1A();
    
  // Trial generator
  TrialGeneratorISR* trialGenPtr = trialPtr->trialGenPtrSav[iTrial];
  int trialGenStatInt            = trialGenPtr->statInt();
    
  // IDs
  int idA = trialPtr->old1.id();
  int idK = trialPtr->old2.id();
    
  // Are we forcing a splitting?
  bool forceSplitting = trialPtr->forceSplittingSav;
  if (forceSplitting) {
    // Set zMin and zMax accordingly
    trialPtr->zMinSav[iTrial] = 
      trialPtr->trialGenPtrSav[iTrial]->getZmin(q2new, trialPtr->s(), trialPtr->e1(),
        is1A ? eAused : eBused);
    trialPtr->zMaxSav[iTrial] = 
      trialPtr->trialGenPtrSav[iTrial]->getZmax(q2new, trialPtr->s(), trialPtr->e1(),
        is1A ? eAused : eBused);
    if (verbose >= 9) cout<<" Note: this is a forced splitting"<<endl;
  }
  
  //************************************************************************
  // Generate zeta variable, work out saj, sjk and check initial energies
  //************************************************************************
    
  double saj, sjk;
  bool pass = trialPtr->genTrialInvariants(saj, sjk, is1A ? eAused : eBused,
    iTrial);
  if (!pass) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF",
      "z outside physical range, returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }
    
  // Check the energies of the initial guy
  double sAK = trialPtr->s();
  double sak = sAK + sjk - saj;
  double eA  = trialPtr->e1();
  double ea  = eA*(sAK+sjk)/sAK;
  double eAotherSys = 0.0;
  double eBeamNow   = 0.0;
  if (is1A) {
    for (int i=0; i<(int)initialA.size(); i++) 
      if (i!=iSysWin) eAotherSys+=initialA[i].e();
    eBeamNow = eBeamA;
  }
  else {
    for (int i=0; i<(int)initialB.size(); i++) 
      if (i!=iSysWin) eAotherSys+=initialB[i].e();
    eBeamNow = eBeamB;
  }
  if ( (ea+eAotherSys) > 0.98*eBeamNow ) {
    if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF",
      "Energy of incoming guys exceed beam energy, returning.");
    if (forceSplitting) { // Lowering sjk (splitting orderd in saj) until success
      if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF",
        "Forced splitting, lowering sjk now.");
      ea  = (0.98*eBeamNow - eAotherSys);
      sjk = sAK*(ea-eA)/eA;
      if (sjk <= 0.001) { // This should not happen!
        if (sjk <= 0.0) sjk = pow(10.0,-10.0);
        if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF","Need to choose sjk = " + 
          num2str(sjk) + " probably due to MPI");
      }
      ea  = eA*(sAK+sjk)/sAK;
      sak = sAK + sjk - saj;
      if (sak <= 0.0) {
        sak = 1.0;
        do {
          sak = sak/10.0;
          saj = sAK + sjk - sak;
        } while (saj <= sak);
      }
      qNew  = trialGenPtr->getQ2(saj,sjk,sAK);
      q2new = pow2(qNew);
      trialPtr->scaleSav[iTrial] = qNew;
    }
    else {
      trialPtr->nHull++;
      nTrialsHull++;
      return false;
    }
  }

  //************************************************************************
  // Set flavors and masses
  //************************************************************************
    
  // NOTE in gluon conversion the id of the quark that ends up as the initial
  // guy gets set in the trial generator when evaluating the pdf ratio, so we
  // have to do that here

  BeamParticle* beamAPtrNow = (is1A ? beamAPtr : beamBPtr);
  double PDFscale = q2new;
  double pdfRatio = 1.0;
  if (trialGenStatInt < 11)
    // Always use PDFs in soft if combining trial generators for gluon emission
    pdfRatio = trialIFSoft.physPDFratio(beamAPtrNow, beamBPtr, iSysWin, false,
      idA, idK, eA, eA, ea, ea, PDFscale, PDFscale);
  else pdfRatio = trialGenPtr->physPDFratio(beamAPtrNow, beamBPtr, iSysWin, false,
    idA, idK, eA, eA, ea, ea, PDFscale, PDFscale);
  trialPtr->addPDF(iTrial, pdfRatio); // save
  int gConvID = trialGenPtr->getGconvID();

  // Gluon emission: inherit parent flavors and add middle gluon 
  if (iAntPhys == iQQemitIF || iAntPhys == iQGemitIF || 
    iAntPhys == iGQemitIF || iAntPhys == iGGemitIF) {
    trialPtr->new1.id(idA);
    trialPtr->new2.id(21);
    trialPtr->new3.id(idK);
    trialPtr->new1.m(trialPtr->old1.m());
    trialPtr->new2.m(0.0);
    trialPtr->new3.m(trialPtr->old2.m());
  }
    
  // Gluon splitting in the initial state:
  else if (iAntPhys == iQXsplitIF) {
    trialPtr->new1.id(21);
    trialPtr->new2.id(-idA);
    trialPtr->new3.id(idK);
    trialPtr->new1.m(0.0);
    trialPtr->new2.m(trialPtr->old1.m());
    trialPtr->new3.m(trialPtr->old2.m());
  }
    
  // Gluon conversion in the initial state:
  else if (iAntPhys == iGXconvIF) {
    trialPtr->new1.id(gConvID);
    trialPtr->new2.id(gConvID);
    trialPtr->new3.id(idK);
    trialPtr->new1.m(0.0);
    trialPtr->new2.m(0.0);
    trialPtr->new3.m(trialPtr->old2.m());
    if (verbose == 9) cout<<" gluon backwards evolving to id = "<<gConvID<<"; idK = "<<idK<<" mK = "<<trialPtr->new3.m()<<endl;
  }

  // Gluon splitting in the final state:
  else if (iAntPhys == iXGsplitIF) {
    // Set flavor of splitting
    double nF       = min((int)trialPtr->getColFac(iTrial),nGluonToQuarkF);
    int splitFlavor = int(rndmPtr->flat() * nF) + 1;
    // Check phase space: sQQ = q2new > 4m^2
    int nFmax       = (int)nF;
    if (q2new > 4.0*pow2(mb)) nFmax = min(nFmax,5);
    else if (q2new > 4.0*pow2(mc)) nFmax = min(nFmax,4);
    else if (q2new > 4.0*pow2(ms)) nFmax = min(nFmax,3);
    else nFmax = min(nFmax,2);
    if (nFmax < splitFlavor) return false;
    trialPtr->new1.id(idA);
    // XG->XQQbar where Q is the emission because a col line emitted
    if ( abs(trialPtr->old1.col()) == abs(trialPtr->old2.col()) ) { // IF is c-c or ac-ac
      trialPtr->new2.id( splitFlavor);
      trialPtr->new3.id(-splitFlavor);
    }
    // XG->XQbarQ where Qbar is the emission because a acol line emitted
    else {
      trialPtr->new2.id(-splitFlavor);
      trialPtr->new3.id( splitFlavor);
    }
    trialPtr->new1.m(trialPtr->old1.m());
    trialPtr->new2.m(0.0);
    trialPtr->new3.m(0.0);
  }
    
  //************************************************************************
  // Check local hadronization veto(s) : inside this Branchelemental
  //************************************************************************
    
  // Require all generated final-state invariants at least above lowest 
  // physical meson mass (FF invariant is only the 23 one)
  // (could in principle check only colour-connected invariants but here
  // just check all 23 invariants regardless of colour connection.
  if (!forceSplitting) { // Only if we do not want to force a splitting
    double mMin23 = vinComPtr->mHadMin(trialPtr->new2.id(), trialPtr->new3.id());
    if (sjk < pow2(1.01*mMin23)) {
      if (verbose >= 5) printOut("SpaceAntShower::generateKinematicsIF",
        "=== Branching Vetoed. m23 < 1.01*mMes.");
      trialPtr->nHadr++;
      nTrialsCutM++;
      return false;
    }
  }
    
  //***********************************************************************
  // Generate full kinematics for this trial branching
  //***********************************************************************

  // Generate random (uniform) phi angle
  double phi = 2 * M_PI * rndmPtr->flat();
    
  // Generate branching kinematics, starting from dipole-antenna parents
  vector<Vec4> pOld, pNew;
  pOld.push_back(trialPtr->old1.p());
  pOld.push_back(trialPtr->old2.p());
  pass = vinCluPtr->map2to3masslessIF(pNew, pOld, sAK, saj, sjk, sak, phi);
  if (!pass && !forceSplitting) {
    if (verbose >= 4) printOut("SpaceAntShower::generateKinematicsIF",
      "Invariants do not match when calculating the momenta, returning.");
    trialPtr->nHull++;
    nTrialsHull++;
    return false;
  }
  // Save momenta
  trialPtr->new1.p(pNew[0]);
  trialPtr->new2.p(pNew[1]);
  trialPtr->new3.p(pNew[2]);
  // Set default polarizations: unpolarized (may be assigned later)
  trialPtr->new1.pol(9); 
  trialPtr->new2.pol(9); 
  trialPtr->new3.pol(9);
    
  if (verbose >= 8) {
    printOut("SpaceAntShower::generateKinematicsIF", "Printing pre-branching momenta");
    for (int i=0; i<2; i++) cout << "  " << pOld[i];
    printOut("SpaceAntShower::generateKinematicsIF", "Printing post-branching momenta");
    for (int i=0; i<3; i++) cout << "  " << pNew[i];
  }
    
  // Note: colour flow is not assigned here, since this requires 
  // knowledge of which is the next global colour tag available in
  // the event.
    
  // Verbose output
  if (verbose >= 6) printOut("SpaceAntShower::generateKinematicsIF","finished --------------");
    
  return true;
    
}

// ******
// 
// acceptTrial : main function to accept or reject a trial branching 
//               after full branching kinematics have been constructed.
//     Checks and corrections that require knowing the full event kinematics
//     belong here.
//     *) Optional: apply beyond-LL (beyond 2->3) improvements
//     *) Optional: match to fixed-order QCD matrix elements
//     *) Optional: post-processing of uncertainty variations
//     *) Final decision whether to Accept or Reject Trial Emission
// 

bool SpaceAntShower::acceptTrial(Event& event, BranchelementalISR* trialPtr,
  int iSysBranch, vector<Particle> partsNow, vector<int> antPos) {

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::acceptTrial","begin --------------");

  // Basic info about trial function and scale
  int iTrial     = indxWin;
  if (iTrial < 0) return false;
  double qNew    = trialPtr->getTrialScale(iTrial);
  double q2new   = pow2(qNew);
  int iAntPhys   = trialPtr->getPhysIndex(iTrial);
  bool isII      = trialPtr->isII();
  bool isSwapped = (isII ? trialPtr->getIsSwapped(iTrial) : false);
  // Set to false for IF because there it was always guy 1 how did
  // gluon splitting / conversion in the initial state

  // Trial generator
  TrialGeneratorISR* trialGenPtr = trialPtr->trialGenPtrSav[iTrial];
  int trialGenStatInt            = trialGenPtr->statInt();

  // Parents
  int id1 = trialPtr->old1.id();
  int id2 = trialPtr->old2.id();

  // Invariants
  double S12 = trialPtr->s();
  double s1j = trialPtr->s12();
  double sj2 = trialPtr->s23();

  //**********************************************************************
  // Helicity of mother partons 
  //**********************************************************************

  // First mother is always initial. Second mother is initial for II.
  double hA = trialPtr->old1.pol();
  double hB = trialPtr->old2.pol();
  bool isPolarized = polarizedSys[iSysBranch] && (hA != 9 && hB != 9);

  // Are we doing the combination for gluon emission?
  bool doAcceptCombi = false;
  if (isII && trialGenStatInt < 3) doAcceptCombi = true;
  else if (!isII && trialGenStatInt < 11) doAcceptCombi = true;

  //**********************************************************************
  // Compute spin-summed trial antenna*PDF sum
  // including colour and headroom factors
  //**********************************************************************

  // Sum over individual trial terms
  double antPDFtrialSum = 0.0;
  if (!doAcceptCombi) {
    // No combination: Take antenna only from trial generator that won
    antPDFtrialSum = trialPtr->headRoomSav[iTrial] *
      trialGenPtr->aTrial(s1j, sj2, S12) *
      trialPtr->trialPDFratioSav[iTrial];
  } else {
    // Combination: Take all emission antennae
    double E1old = trialPtr->e1();
    double e1new = trialPtr->new1.e();
    double E2old = trialPtr->e2();
    double e2new = trialPtr->new3.e();
    // Sum over modified trial antenna function * trial pdf ratio
    // Check if headroom factor was actually 1
    for (int iSum = 0; iSum < (int)trialPtr->nTrialGenerators(); ++iSum) {
      // Only include terms that correspond to the current physical antenna
      if (trialPtr->getPhysIndex(iSum) != iAntPhys) continue;
      TrialGeneratorISR* tGPtr = trialPtr->trialGenPtrSav[iSum];
      // for H->gg soft trial did the job
      double headRoomFac  = 
        getHeadRoom(tGPtr->statInt(), -1, nG[iSysWin], nQQ[iSysWin],
          iSysWin, qNew) ;
      double aPDFtrial = ( tGPtr->aTrialMod(s1j, sj2, S12, E1old, E2old, e1new, e2new) *
        tGPtr->trialPDFratio(beamAPtr, beamBPtr, iSysWin, false, id1, id2, 
          E1old, E2old, e1new, e2new, true) );
      antPDFtrialSum += (headRoomFac*aPDFtrial);
    } 
  }
  // Colour factor, we used the physical one from the antenna in pTnext
  antPDFtrialSum *= getAnt(iAntPhys)->getChargeFactor();

  //**********************************************************************
  // If physical antenna function is mirror of current trial, translate to
  // swapped invariants for antenna-function evaluation
  //**********************************************************************

  double hAant  = hA;
  double hBant  = hB;
  double s1jant = s1j;
  double sj2ant = sj2;
  if ( isSwapped ) {
    hAant  = hB;
    hBant  = hA;
    s1jant = sj2;
    sj2ant = s1j;
  }

  //***********************************************************************
  // AlphaS
  // Impose default choice. Can differ slighly from trial even when running
  // inside trial integral, due to flavor thresholds. Here, alphaS(mu) is
  // returned directly, with the number of flavors active at mu, whereas 
  // the number of flavors in the trial integral is controlled by the value
  // of the trial scale.
  //***********************************************************************

  double alphaTrial = trialPtr->getAlphaTrial(iTrial);
  if (isnan(alphaTrial)) cout << " alphaStrial is NAN!!!" << endl;
  if (alphaTrial != alphaTrial) cout << " alphaStrial is != alphaStrial" << endl;
  // Scale is pT for gluon emission and m for splitting / conversion
  double mu2     = (isII ? s1j*sj2/(S12+s1j+sj2) : s1j*sj2/(S12+sj2));
  double kMu2Usr = aSkMu2;
  if (iAntPhys == iXGsplitIF) {
    if (!evAllPT) mu2 = sj2;
    kMu2Usr = aSkMu2SplitF;
  } else if (iAntPhys == iQXsplitIF || iAntPhys == iQXsplitII) {
    kMu2Usr = aSkMu2SplitI;
    if (!evAllPT) mu2 = (isII ? s1jant : s1j);
  } else if (iAntPhys == iGXconvIF || iAntPhys == iGXconvII) {
    kMu2Usr = aSkMu2Conv;
    if (!evAllPT) mu2 = (isII ? s1jant : s1j);
  }
  double mu2Usr    = max(mu2min, mu2freeze + mu2*kMu2Usr);
  // alphaS values
  double alphaSusr = min(alphaSmax, alphaSptr->alphaS(mu2Usr));

  // Verbose output
  if (verbose >= 8)
    printOut("SpaceAntShower::acceptTrial", "alphaTrial = "
      + num2str(alphaTrial) + ", alphaSusr = " + num2str(alphaSusr));

  //************************************************************************
  // Compute spin-summed physical antennae (spin selection below)
  //************************************************************************

  // Define pointer antennae
  AntennaISR* antPtr = getAnt(iAntPhys);

  // Compute spin-summed antenna function 
  if (verbose >= 8) 
    printOut("SpaceAntShower::acceptTrial","Evaluating antenna function and PDF ratio");

  double helSum;
  // Unpolarized case: ignore parent helicities
  if (!isPolarized) {
    helSum = antPtr->antennaFunction(s1jant, sj2ant, S12);
  }
  // Polarized case: use parent helicities (but restrict to massless)
  else {
    helSum = antPtr->antennaFunction(s1jant, sj2ant, S12, hAant, hBant, 
      9, 9, 9);
  }

  // Verbose output
  if (verbose >= 8)
    printOut("SpaceAntShower::acceptTrial","helSum  = "+num2str(helSum));

  // Store (partial) helicity sum
  double antPhys = helSum;

  //************************************************************************
  // Apply color (charge) factor
  //************************************************************************

  antPhys *= antPtr->getChargeFactor();

  //************************************************************************
  // PDF factor
  //************************************************************************

  double PDFphys = trialPtr->physPDFratioSav[iTrial];

  // Extra factor from using running of PDFs close to mass threshold
  double extraMassPDFfactor = trialPtr->extraMassPDFfactorSav[iTrial];
  PDFphys *= extraMassPDFfactor;

  //**********************************************************************
  // Compute accept probability : Physical/Trial 
  //**********************************************************************

  Paccept[0] = antPhys*PDFphys/antPDFtrialSum;

  if (verbose >= 8) {
    if (!doAcceptCombi) {
      printOut("SpaceAntShower::acceptTrial", "No combination:");
      cout << "   AntTrial/s    = " << setprecision(6)
           << trialGenPtr->aTrial(s1j, sj2, S12) << endl;
      cout << "   AntPhys/s     = " << setprecision(6)
           << trialGenPtr->aPhys(s1j, sj2, S12, id1, id2) << endl;
      cout << "   massPDFfactor = " << setprecision(6) << extraMassPDFfactor
           << endl;
      cout << "   PDFtrial      = " << setprecision(6)
           << trialPtr->trialPDFratioSav[iTrial] << endl;
      cout << "   PDFphys       = " << setprecision(6)
           << trialPtr->physPDFratioSav[iTrial] << endl;
      cout << "   Paccept       = " << setprecision(6) << Paccept[0] << endl;
    } else {
      printOut("SpaceAntShower::acceptTrial", "Combination (gluon emission):");
      cout << "   AntPDFtrial = " << setprecision(6) << antPDFtrialSum << endl;
      cout << "   AntPhys/s   = " << setprecision(6) << antPhys << endl;
      cout << "   PDFphys     = " << setprecision(6) << PDFphys << endl;
      cout << "   Paccept     = " << setprecision(6) << Paccept[0] << endl;
    }
  }

  //***********************************************************************
  // Choose helicities for daughter particles 
  // (so far only for massless polarized shower)
  //***********************************************************************

  double aHel = 0.0;
  if ( isPolarized ) {
    // Generate random number
    double randHel = rndmPtr->flat() * helSum;
    // Select helicity
    int hi(0), hj(0), hk(0);
    for (hi = hAant; abs(hi) <= 1; hi -= 2*hAant) {
      for (hk = hBant; abs(hk) <= 1; hk -= 2*hBant) {
        for (hj = hAant; abs(hj) <= 1; hj -= 2*hAant) {
          aHel = antPtr->antennaFunction(s1jant, sj2ant, S12, hAant, hBant,
            hi, hj, hk);
          randHel -= aHel;
          if (verbose >= 8)
            printOut("SpaceAntShower::acceptTrial", "antPhys("+
              num2str(int(hAant))+" "+num2str(int(hBant))+"  -> "+num2str(hi)+
              " "+num2str(hj)+" "+num2str(hk)+") = "+num2str(aHel)+
              ", isSwapped = "+bool2str(isSwapped)+"; sum = "+num2str(helSum));
          if (randHel < 0.) break;
        }
        if (randHel < 0.) break;
      }
      if (randHel < 0.) break;
    }
    if (verbose >= 7) 
      printOut("SapceAntShower::acceptTrial", "selected"+num2str(int(hAant))+
        " "+num2str(int(hBant))+"  -> "+num2str(hi)+" "+num2str(hj)+" "+
        num2str(hk)+", isSwapped = "+bool2str(isSwapped)+"\n");
    // Assign helicities (taking swapped invariants into account)
    if (!isSwapped) {
      //cout<<" Selected : " <<hA<<" "<<hB<<" > "<<hi<<" "<<hj<<" "<<hk<<endl;
      trialPtr->new1.pol(hi);
      trialPtr->new2.pol(hj);
      trialPtr->new3.pol(hk);
    } 
    else {
      //cout<<" Selected : " <<hB<<" "<<hA<<" > "<<hk<<" "<<hj<<" "<<hi<<endl;
      trialPtr->new1.pol(hk);
      trialPtr->new2.pol(hj);
      trialPtr->new3.pol(hi);        
    }
  }
  // If not polarized
  else {      
    trialPtr->new1.pol(9);
    trialPtr->new2.pol(9);
    trialPtr->new3.pol(9);
    polarizedSys[iSysBranch] = false;
  }

  // Store chosen polarizations also in vector representing system state
  partsNow[antPos[0]].pol(trialPtr->new1.pol());
  partsNow[antPos[2]].pol(trialPtr->new2.pol());
  partsNow[antPos[1]].pol(trialPtr->new3.pol());

  //**********************************************************************
  // Apply alphaS choices
  //**********************************************************************

  Paccept[0] *= alphaSusr / alphaTrial;

  //**********************************************************************
  // Matrix Element Corrections
  //**********************************************************************

  // QCD order as in Born*QCD
  int nQCDnow = nBranch[iSysWin]+1;
  // Only up to matchingLO
  bool doMEC = (meCorrectionsNow && nQCDnow <= matchingLO);
  // Only for hard process
  bool isHardNow = hardSys[iSysWin];
  if (!isHardNow) doMEC = false;
  // We are already below the cutoff (only checking for last order)
  if ( (nQCDnow == matchingLO) && (q2new < pow2(matchingIRcutoff)) ) 
    doMEC = false;

  // Current number of quarks and gluon
  int nGnow  = nG[iSysWin];
  int nQQnow = nQQ[iSysWin];
  if (iAntPhys == iGXconvII || iAntPhys == iGXconvIF || iAntPhys == iXGsplitIF) {
    ++nQQnow;
    nGnow -= 1;
  } else ++nGnow;

  // Compute matching scale dampening factor (if imposed)
  double DeltaDamp = 1.0;
  double q2damp    = q2new;
  double q2match   = q2fac;
  if (doMEC && nQCDnow >= matchingRegOrder && matchingRegOrder != 0) {
    // If matching scale is abolute use given scale
    if (matchingScaleIsAbs) q2match = pow2(matchingScale);
    // If matching scale is relative use given ratio
    else q2match = pow2(matchingScaleRatio)*q2match;
    // MatchingRegShape = 0: Step Function
    if (matchingRegShape == 0) {
      if (q2damp < q2match) {
        DeltaDamp = 0.0;
        doMEC = false;
      }
      else DeltaDamp = 1.0;
    }
    // MatchingRegShape = 1: linear in log(Q) between 1/2 QM2 < Q2 < 2 QM2
    else {
      double log2R = log(q2damp/q2match)/log(2.);
      if (log2R < -1) {
        DeltaDamp = 0.0;
        doMEC = false;
      } else if (log2R > 1.) {
        DeltaDamp = 1.0;
      } else {            
        DeltaDamp = 0.5*log2R + 0.5 ;
      }
    }
  }

  // Set all relevant properties and check if we can do the MEC
  if (doMEC) {
    matrixElementCorrectionsPtr->prepMECs(MEclass[iSysWin], nQLneeded[iSysWin],
      Hint[iSysWin]);
    doMEC = matrixElementCorrectionsPtr->canDoMEC(partsNow, isHardNow);
  }

  // Compute MEC and apply to accept probability
  if (doMEC) {
    // Copy event
    Event eventNew = event;
    // Only keep system and beams
    eventNew.remove(3,eventNew.size()-1);
    // Append the incoming partons, reset relations to beams
    for (int i=0; i<(int)partsNow.size(); i++) if (!partsNow[i].isFinal())
      eventNew.append(partsNow[i]);
    eventNew[1].daughters(3,0); eventNew[2].daughters(4,0);
    eventNew[3].mothers(1,0);   eventNew[4].mothers(2,0);
    // Now add all final state particles
    for (int i=0; i<(int)partsNow.size(); i++) if (partsNow[i].isFinal())
      eventNew.append(partsNow[i]);
    // If no merging cluster only shower jets
    int nClustStep = ( doMergeSav ? -1 : nQCDnow-additionalJets[iSysWin] );
    PME = matrixElementCorrectionsPtr->performMECs(eventNew, FSRshowerPtr,
      this, nClustStep);
    // Modify accept probability
    Paccept[0] *= 1.0 + DeltaDamp * (PME - 1.0);
  }

  //**********************************************************************
  // Print MC violations
  //**********************************************************************

  if (verbose >= 8)
    printOut("SpaceAntShower::acceptTrial"," Trial Paccept = "
      +num2str(Paccept[0]));

  bool violation  = (Paccept[0] > 1.0 + TINY);
  bool negPaccept = (Paccept[0] < 0.0);

  if (violation || negPaccept) {
    int indexAccProb;
    if (meCorrectionsNow && nQCDnow <= matchingLO && isHardNow)
      indexAccProb = ( doMEC ? 1 : 2 );
    else indexAccProb = 0;
    if (violation) {
      nMCviols[indexAccProb]++;
      maxMCviols[indexAccProb] = max(maxMCviols[indexAccProb],Paccept[0]);
      // Automatically increase headroom factor when violations encountered
      setHeadRoom( trialGenStatInt, -1, nG[iSysWin], nQQ[iSysWin],
        iSysWin, qNew, min(1.2,Paccept[0]) );
    } else if (negPaccept) {
      nNegPas[indexAccProb]++;
      minNegPas[indexAccProb] = min(minNegPas[indexAccProb],Paccept[0]);
    }
    bool report = false;
    bool isLast = false;
    // Report violations if verbose >= 3 or verbose >= 1 && fussy or
    // verbose >= 1 && negative Paccept, for first nReportMCviol times
    if ( (verbose >= 1 && Paccept[0] > pAcceptFuss && qNew > pAcceptPT0) ||
      (verbose >= 1 && negPaccept) || verbose >= 3 ) {
      nReportedMCviol++;
      if (nReportedMCviol <= nReportMCviol) {
        report = true;
        if (nReportedMCviol == nReportMCviol) isLast = true;
      }
    }
    // Always report in debugging mode
    if (report || verbose >= 4) {
      printOut("SpaceAntShower::acceptTrial",
        (doMEC ? " Paccept(shower and ME, order" : " Paccept(shower, order")
        +num2str(nQCDnow)+") = "+num2str(Paccept[0])+" at q = "
        +num2str(qNew)+" in Trial Generator "+trialGenPtr->name());
      if (extraMassPDFfactor != 1.0) 
        cout <<" == Note: PDFs close to mass threshold"<<endl;
      if (verbose >= 4 || Paccept[0]>100.0) {
        if (doMEC)
          cout <<" bare PME = "<<PME<<" qDamp = "<<sqrt(q2damp)<<" qDamp/qMatch = 1/"
               <<sqrt(q2match/q2damp)<<" DeltaDamp = "<<DeltaDamp<<endl;
        cout <<(isII ? " AB -> ajb = " : " AK -> ajk = ")<<trialPtr->old1.id()
             <<" "<<trialPtr->old2.id()<<" -> "<<trialPtr->new1.id()<<" "
             <<trialPtr->new2.id()<<" "<<trialPtr->new3.id()<<" maj = "
             <<sqrt(m2(trialPtr->new1.p()+trialPtr->new2.p()))<<(isII ? " mjb" :
             " mjk")<<" = "<<sqrt(m2(trialPtr->new2.p()+trialPtr->new3.p()))
             <<(isII ? " mAB" : " mAK")<<" = "<<sqrt(S12)<<endl;
      }
      if (isLast && verbose < 4) cout <<" Further warnings suppressed!"<<endl;
      // Print more info for bad cases
      if (verbose >= 5) trialPtr->list();
      if ((Paccept[0]>2.0 && verbose>=4) || (Paccept[0]>10.0 && verbose>=3)) {
        cout << " Momentum Listing after the branching: " << endl;
        for (int i=0; i<(int)partsNow.size(); i++) {
          cout <<" id"<<i<<(i>9 ? " = " : "  = ")<<num2str(partsNow[i].id(),7);
          if (polarizedSys[iSysBranch])
            cout <<", pol = "<<num2str((int)partsNow[i].pol(),3);
          cout <<", p = "<<partsNow[i].p();
        }
        cout << " Momentum Listing before the branching: " << endl;
        for (int i=0; i<(int)partsSav[iSysWin].size(); i++) {
          cout <<" id"<<i<<(i>9 ? " = " : "  = ")
               <<num2str(partsSav[iSysWin][i].id(),7);
          if (polarizedSys[iSysBranch])
            cout <<", pol = "<<num2str((int)partsSav[iSysWin][i].pol(),3);
          cout <<", p = "<<partsSav[iSysWin][i].p();
        }
      }
      if (Paccept[0] > 100.0) {
        int    nResc        = trialPtr->getNshouldRescue(iTrial);
        double PDFphysTrial = trialPtr->physPDFratioSav[iTrial]/
          trialPtr->trialPDFratioSav[iTrial];
        if (nResc>0 || PDFphysTrial>100.0) {
          if (nResc>0) cout << " Had " << nResc << " pTnext ~ pTold,";
          else cout << " Headroom = " << trialPtr->headRoomSav[iTrial]
             << ", PDFphys/ PDFtrial = " << PDFphysTrial;
          cout << endl;
        }
      }
    }
  }

  //**********************************************************************
  // MC reweighting and uncertainty bands
  //**********************************************************************

  pair<double,double> accRecWeight = weightsPtr->doMCreweighting(Paccept);
  double acceptWeight = accRecWeight.first;
  double rejectWeight = accRecWeight.second;

  if (uncertaintyBands) {
    weightsPtr->doVariationsISR(Paccept, isHardNow, doMEC, trialPtr,
      iTrial, alphaSptr, alphaSusr, mu2*kMu2Usr, mu2min, mu2freeze, alphaSmax,
      isPolarized, antPhys/antPtr->getChargeFactor(), helSum, aHel);
  }

  //**********************************************************************
  // Accept / Reject step
  //**********************************************************************

  if (rndmPtr->flat() > Paccept[0]) {
    if (verbose >= 5)
      printOut("SpaceAntShower::acceptTrial",
        "=== Branching Vetoed. wPhys/wTrial = " + num2str(Paccept[0]));

    // In case of reweighting or uncertainties, scale the weight(s)
    if (allowReweighting) weightsPtr->scaleWeightAll(rejectWeight);
    if (uncertaintyBands) weightsPtr->scaleWeightVar(Paccept, false, isHardNow,
      "ISR", trialGenStatInt);

    // For each failed trial, attempt to (slowly) optimize headroomfactors
    if ( trialPtr->nVeto >= 1 && optimizeHeadRoom ) 
      setHeadRoom( trialGenStatInt, -1, nG[iSysWin],
        nQQ[iSysWin], iSysWin, qNew, 0.9999 );

    // Count up number of vetoed branchings
    trialPtr->nVeto++;
    nTrialsVeto++;
    return false;
  }

  // In case of reweighting or uncertainties, scale the weight(s)
  if (allowReweighting) weightsPtr->scaleWeightAll(acceptWeight);
  if (uncertaintyBands) weightsPtr->scaleWeightVar(Paccept, true, isHardNow,
      "ISR", trialGenStatInt);

  // Verbose output
  if (verbose >= 6) 
    printOut("SpaceAntShower::acceptTrial","finished --------------");

  // All clear, yey
  return true;

}
  
// ******
// 
// assignColourFlow():
// 
// Paint a trialPtr with colour-flow information,
// according to the type of branching.
  
bool SpaceAntShower::assignColourFlow(Event& event, BranchelementalISR* trialPtr) {
  bool usedColTag = false;
    
  // Basic info about trial function and old partons
  int iTrial     = indxWin;
  int iAntPhys   = trialPtr->getPhysIndex(iTrial);
  bool isSwapped = trialPtr->getIsSwapped(iTrial);
  int colOld     = trialPtr->col;
  int col1       = trialPtr->old1.col();
  int acol1      = trialPtr->old1.acol();
  int col2       = trialPtr->old2.col();
  int acol2      = trialPtr->old2.acol();
    
  // Gluon emission:
  if (trialPtr->new2.id() == 21) {
    double s12 = dot4(trialPtr->new1.p(),trialPtr->new2.p());
    double s23 = dot4(trialPtr->new2.p(),trialPtr->new3.p());
    // Who inherits the colour?
    bool inh12 = reconnectionsPtr->inherit12(s12,s23);
    // NOTE: Use lastColTag here, if branching accepted we tell
    // Pythia that we used one
    int nTag   = event.lastColTag()+1;
    usedColTag = true;
    // Check other colourtag for gluons
    int colL   = 0;
    int colR   = 0;
    if (trialPtr->iL != 0) {
      if (colOld == col1) colL = trialPtr->old1.acol();
      else                colL = trialPtr->old1.col();
    }
    if (trialPtr->iR != 0) {
      if (colOld == col2) colR = trialPtr->old2.acol();
      else                colR = trialPtr->old2.col();
    }
    int nextTag = 10*int(nTag/10) + 10;
    int colNew  = nextTag + int(colOld%10 + rndmPtr->flat()*8)%9 + 1;
    if (inh12) { // new1 inherits colour
      while (colNew%10 == colR%10) 
        colNew = nextTag + int(colOld%10 + rndmPtr->flat()*8)%9 + 1;  
      trialPtr->new1.acol(acol1);
      trialPtr->new1.col(col1);
      trialPtr->new2.acol(colOld == col1 ? colNew : colOld);
      trialPtr->new2.col(colOld == col1 ? colOld : colNew);
      trialPtr->new3.acol(colOld == acol2 ? colNew : acol2);
      trialPtr->new3.col(colOld == acol2 ? col2 : colNew);
    } else { // new3 inherits colour
      while (colNew%10 == colL%10) 
        colNew = nextTag + int(colOld%10 + rndmPtr->flat()*8)%9 + 1;
      trialPtr->new1.acol(colOld == col1 ? acol1 : colNew);
      trialPtr->new1.col(colOld == col1 ? colNew : col1);
      trialPtr->new2.acol(colOld == col1 ? colOld : colNew);
      trialPtr->new2.col(colOld == col1 ? colNew : colOld);
      trialPtr->new3.acol(acol2);
      trialPtr->new3.col(col2);
    }
  }
    
  // Gluon splitting in the initial state: side A
  else if ((iAntPhys == iQXsplitII && !isSwapped) || iAntPhys == iQXsplitIF) {
    // II new1 G -> old1 Q/QB (enters hard process) + new2 QB/Q and
    //    new3   -> old2 as recoiler
    // NOTE: Use lastColTag here, if branching accepted we tell
    // Pythia that we used one
    int nTag   = event.lastColTag()+1;
    usedColTag = true;
    if (colOld == col1) { // Splitting quark -> antiquark in FS
      trialPtr->new1.acol(nTag);
      trialPtr->new1.col(col1);
      trialPtr->new2.acol(nTag);
      trialPtr->new2.col(0);
    }
    else { // Splitting antiquark -> quark in FS
      trialPtr->new1.acol(acol1);
      trialPtr->new1.col(nTag);
      trialPtr->new2.acol(0);
      trialPtr->new2.col(nTag);
    }
    trialPtr->new3.acol(acol2);
    trialPtr->new3.col(col2);
  }
    
  // Gluon splitting in the initial state: side B
  else if (iAntPhys == iQXsplitII && isSwapped) {
    // II new3 G -> old2 Q/QB (enters hard process) + new2 QB/Q and
    //    new1   -> old1 as recoiler
    // NOTE: Use lastColTag here, if branching accepted we tell
    // Pythia that we used one
    int nTag   = event.lastColTag()+1;
    usedColTag = true;
    if (colOld == col2) { // Splitting quark -> antiquark in FS
      trialPtr->new2.acol(nTag);
      trialPtr->new2.col(0);
      trialPtr->new3.acol(nTag);
      trialPtr->new3.col(col2);
    }
    else { // Splitting antiquark -> quark in FS
      trialPtr->new2.acol(0);
      trialPtr->new2.col(nTag);
      trialPtr->new3.acol(acol2);
      trialPtr->new3.col(nTag);
    }
    trialPtr->new1.acol(acol1);
    trialPtr->new1.col(col1);
  }
    
  // Gluon convsersion in the initial state: side A
  else if ((iAntPhys == iGXconvII && !isSwapped) || iAntPhys == iGXconvIF) {
    // II new1 Q/QB -> old1 G (enters hard process) + new2 Q/QB and
    //    new3      -> old2 as recoiler
    if (trialPtr->new2.id() > 0) { // quark
      trialPtr->new1.acol(0);
      trialPtr->new1.col(col1);
      trialPtr->new2.acol(0);
      trialPtr->new2.col(acol1);
    }
    else { // antiquark
      trialPtr->new1.acol(acol1);
      trialPtr->new1.col(0);
      trialPtr->new2.acol(col1);
      trialPtr->new2.col(0);
    }
    trialPtr->new3.acol(acol2);
    trialPtr->new3.col(col2);
  }
    
  // Gluon conversion in the initial state: side B
  else if (iAntPhys == iGXconvII && isSwapped) {
    // II new3 Q/QB -> old2 G (enters hard process) + new2 Q/QB and
    //    new1      -> old1 as recoiler
    if (trialPtr->new2.id() > 0) { // quark
      trialPtr->new2.acol(0);
      trialPtr->new2.col(acol2);
      trialPtr->new3.acol(0);
      trialPtr->new3.col(col2);
    }
    else { // antiquark
      trialPtr->new2.acol(col2);
      trialPtr->new2.col(0);
      trialPtr->new3.acol(acol2);
      trialPtr->new3.col(0);
    }
    trialPtr->new1.acol(acol1);
    trialPtr->new1.col(col1);
  }

  // Gluon splitting in the final state:
  else if ( iAntPhys == iXGsplitIF) {
    // IF old2 G -> new3 QB/Q + new2 Q/QB and
    //    new1   -> old1 as recoiler
    if (trialPtr->new2.id() > 0) { // quark
      trialPtr->new2.acol(0);
      trialPtr->new2.col(col2);
      trialPtr->new3.acol(acol2);
      trialPtr->new3.col(0);
    }
    else { // Splitting acol line -> emission is antiquark
      trialPtr->new2.acol(acol2);
      trialPtr->new2.col(0);
      trialPtr->new3.acol(0);
      trialPtr->new3.col(col2);
    }
    trialPtr->new1.acol(acol1);
    trialPtr->new1.col(col1);
  }
    
  return usedColTag;
}
  
//********* 
  
// Print header information
void SpaceAntShower::header() {
    
  // Avoid printing header several times
  headerIsPrinted = true;
    
  cout << setprecision(3);
  cout.setf(ios::left); 
      
  // Print header information about Antennae
  if (verbose >= 2) {
    cout<<" | \n";
    for (int iAnt=0;iAnt<=11;++iAnt) {
      cout.setf(ios::left);
      cout<<setprecision(2);
      cout<<" | *------------------------------------------------------------------------\n";
      cout<<" | * "<<getAnt(iAnt)->vinciaName()<<endl;
      double chargeFactor = getAnt(iAnt)->getChargeFactor();
      if (chargeFactor > 0.) { 
        // Print color/charge factor
        cout<<" |   chargeFactor       = "<<fixed
            <<setw(6)<<chargeFactor<<"\n";
        cout << " | \n";
      } else { // Switched off
        cout<<" |   OFF \n | \n";
      }
    }
    cout << " *-------  End VINCIA ISR Global Initialization  "
         <<"-----------------------------------------*\n\n";
  }
      
}
  
// Print the list of dipoles.
void SpaceAntShower::list() const {
  // Loop over dipole list and print it.
  for (int iSys = 0; iSys < (int)branchelementals.size(); iSys++) {
    for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
      if (branchelementals[iSys].size() == 1) branchelementals[iSys][iAnt].list(true,true);
      else if ( iAnt == 0 ) branchelementals[iSys][iAnt].list(true,false);
      else if ( iAnt == int(branchelementals[iSys].size())-1 ) branchelementals[iSys][iAnt].list(false,true);
      else branchelementals[iSys][iAnt].list();
    }
  }
}

void SpaceAntShower::printInfo(bool pluginCall) {
  if (!isInit) {
    if (pluginCall)
      cout << " *--------  End VINCIA FSR and ISR Statistics  --------------------------------------------------------------------*\n\n";
    return;
  }
    
  // Weight info
  int nTotWeightsInt         = weightsPtr->nTotWeights;
  int nWeights               = weightsPtr->nVariations();
  int nNonunityInitialWeight = weightsPtr->nNonunityInitialWeight;
  int nNegativeInitialWeight = weightsPtr->nNegativeInitialWeight;
  int nNonunityWeight        = weightsPtr->nNonunityWeight;
  int nNegativeWeight        = weightsPtr->nNegativeWeight;
  double nTotWeights         = max(1.0,((double)(nTotWeightsInt)));
  vector<double> weightSum   = weightsPtr->weightSum;
  vector<double> weightMax   = weightsPtr->weightsMax;
  vector<double> weightMin   = weightsPtr->weightsMin;
    
  if (pluginCall)
    cout<< " | - - - -  ISR only Statistics  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - |\n";
  else {
    cout<< "\n";
    cout<< " *--------  VINCIA ISR Statistics  --------------------------------------------------------------------------------*\n";
    cout << " |                                                                                                                 |\n"; 
    cout << " |                                                                        "<<setw(40)<<" "<<" |\n";
    cout << " | Total Number of (accepted) events                           = "
         <<num2str(nTotWeightsInt,9)<<setw(40)<<" "<<" |\n";
    cout << " | Number of events with nonunity initial weight               = "
         <<( (nNonunityInitialWeight <= 0) ? "     none                                " 
           : num2str(nNonunityInitialWeight,9)+" <-- (INITIALLY) WEIGHTED EVENTS" )
         <<setw(8)<<" "<<" |\n";
    cout << " | Number of events with negative initial weight               = "
         <<( (nNegativeInitialWeight <= 0) ? "     none" 
           : num2str(nNegativeWeight,9))
         <<setw(40)<<" "<<" |\n";
    cout << " | Number of events with nonunity reweight                     = "
         <<( (nNonunityWeight <= 0) ? "     none                      " 
           : num2str(nNonunityWeight,9)+" <-- REWEIGHTED EVENTS" )
         <<setw(18)<<" "<<" |\n";
    cout << " | Number of events with negative reweight                     = "
         <<( (nNegativeWeight <= 0) ? "     none" 
           : num2str(min(nNegativeWeight,nNonunityWeight),9))
         <<setw(40)<<" "<<" |\n";
    cout << " |                                                                        "<<setw(40)<<" "<<" |\n";   
    cout << " |                      weight(i)          Avg Wt   Avg Dev  rms(dev)      kUnwt          Expected effUnw          |\n";
    cout << " | This run               i =     IsUnw       <w>     <w-1>                1/<w>        Max Wt  <w>/MaxWt   Min Wt |\n";
    cout << " |"<<setw(4)<<" "<<"User settings                 0 "; 
    cout << ( (abs(1.0-weightSum[0]/nTotWeights) < TINY) ? "   yes " : "    no " );
    cout << num2str(weightSum[0]/nTotWeights,9)<<" ";
    cout << num2str(weightSum[0]/nTotWeights-1.0,9)<<" ";
    cout << setw(9)<<"-"<<"  ";
    cout << ((weightSum[0] != 0.0) ? num2str(nTotWeights/weightSum[0],9): num2str(0.0,9));
    cout << num2str(weightMax[0],14)<<" ";
    cout << ((weightMax[0] != 0.0) ? num2str(weightSum[0]/nTotWeights/weightMax[0],9) : num2str(0.0,9))<<" ";
    cout << num2str(weightMin[0],9)<<" ";
    cout << "|\n";
    if (uncertaintyBands && nWeights > 1) {
      for (int iVar=1; iVar<nWeights; iVar++) {
        cout << " |"<<setw(4)<<" "<<"Var : "<<setw(12)<<weightsPtr->varLabels[iVar-1]<<"     "<<num2str(iVar,3);
        cout <<( (abs(1.0-weightSum[iVar]/nTotWeights) < TINY) ? "    yes " : "     no " );
        cout << num2str(weightSum[iVar]/nTotWeights-1.0,9)<<" ";
        cout << setw(9)<<"-"<<"  ";
        cout << ((weightSum[iVar] != 0.0) ? num2str(nTotWeights/weightSum[iVar],9): num2str(0.0,9));
        cout << num2str(weightMax[iVar],14)<<" ";
        cout << ((weightMax[iVar] != 0.0) ? num2str(weightSum[iVar]/nTotWeights/weightMax[iVar],9): num2str(0.0,9))<<" ";
        cout << num2str(weightMin[iVar],9)<<" ";
        cout << "|\n";
      }
    }
  }
  cout<< " |                                                                        "<<setw(40)<<" "<<" |\n";
  if (verbose >= 2 && nTrialsSum > 0) {
    cout<< " | Trial Veto Rates:"<<fixed<<setprecision(3)
        <<"   Hull = "<<nTrialsHull*1.0/nTrialsSum
        <<"   Massive = "<<nTrialsHullMassive*1.0/nTrialsSum
        <<"   Had M+Q = "<<nTrialsCutM*1.0/nTrialsSum<<" + "<<nTrialsCutQ*1.0/nTrialsSum
        <<"   HQ PS = "<<nClosePSforHQ*1.0/nTrialsSum
        <<"   Veto = "<<nTrialsVeto*1.0/nTrialsSum
        <<"     |"<<endl;
    cout<< " |                                                                        "<<setw(40)<<" "<<" |\n";
  }
  string accProb[3] = { " pure shower,                          ", 
    " with MECs,                            ", " with planned but not carried out MECs," };
  int    nMCviol    = nMCviols[0]+nMCviols[1]+nMCviols[2];
  double maxMCviol  = max(max(maxMCviols[0],maxMCviols[1]),maxMCviols[2]);
  int    nNegPa     = nNegPas[0]+nNegPas[1]+nNegPas[2];
  double minNegPa   = min(min(minNegPas[0],minNegPas[1]),minNegPas[2]);
  if (nMCviol > 0) {
      cout<< " | Encountered "<<setw(6)<<nMCviol<<" MC violations,                         "
          <<"with a maximum violation of "<<setw(15)<<maxMCviol<<setw(10)<<" "<<" |\n";
    for (int i=0; i<3; i++) if (nMCviols[i] > 0)
      cout<< " |             "<<setw(6)<<nMCviols[i]<<accProb[i]<<" with a maximum violation of "
          <<setw(15)<<maxMCviols[i]<<setw(10)<<" "<<" |\n";
    cout<< " |                                                                        "<<setw(40)<<" "<<" |\n";
  }
  if (nNegPa > 0) {
    cout<< " | Encountered "<<setw(6)<<nNegPa<<" negative accept probabilities,         "
        <<"with a minimum value of "<<setw(15)<<minNegPa<<setw(14)<<" "<<" |\n";
    for (int i=0; i<3; i++) if (nNegPas[i] > 0)
      cout<< " |             "<<setw(6)<<nNegPas[i]<<accProb[i]<<" with a minimum value of "
          <<setw(15)<<minNegPas[i]<<setw(14)<<" "<<" |\n";
    cout<< " |                                                                        "<<setw(40)<<" "<<" |\n";
  }
  int nPDFoutsideBound = trialIISoft.getNpdfOB()   + trialIIGCollA.getNpdfOB()  + trialIIGCollB.getNpdfOB()  +
    trialIISplitA.getNpdfOB() + trialIISplitB.getNpdfOB()  + trialIIConvA.getNpdfOB()   +
    trialIIConvB.getNpdfOB()  + trialIFSoft.getNpdfOB()    + trialIFQCollA.getNpdfOB()  + 
    trialIFQCollK.getNpdfOB() + trialIFGCollA1.getNpdfOB() + trialIFGCollA2.getNpdfOB() +
    trialIFGCollK.getNpdfOB() + trialIFSplitA.getNpdfOB()  + trialIFSplitK.getNpdfOB()  +
    trialIFConvA.getNpdfOB();
  if (nPDFoutsideBound > 0) {
    cout<< " | Encountered "<<setw(6)<<nPDFoutsideBound<<" PDF evaluations outside of the boundaries given by the PDF set"
        <<setw(33)<<" |\n";
    cout<< " |                                                                        "<<setw(40)<<" "<<" |\n";
  }
  if (pluginCall)
    cout<< " *--------  End VINCIA FSR and ISR Statistics  --------------------------------------------------------------------*\n\n";
  else
    cout<< " *--------  End VINCIA ISR Statistics -----------------------------------------------------------------------------*\n\n";
}
  
//********* 
    
// Method to check if a gluon splitting in the initial state (to get rid of
// heavy quarks) is still possible after the current branching
bool SpaceAntShower::checkHeavyQuarkPhaseSpace(vector<Particle> parts, int /*iSys*/) {
  // We don't want to get rid of the quarks anyway
  if (massCorrectionLevel == 0) return true;
  vector<int> isToCheck; isToCheck.resize(0);
  for (int i=0; i<(int)parts.size(); i++) if (!parts[i].isFinal()) {
      if (parts[i].idAbs() > nFlavZeroMass) if (parts[i].idAbs() < 6) isToCheck.push_back(i); 
    }
  // Loop over partons to check
  for (int i=0; i<(int)isToCheck.size(); i++) {
    Particle heavyQuark = parts[isToCheck[i]];
    int hQcol           = ( (heavyQuark.col() == 0) ? heavyQuark.acol() : heavyQuark.col() );
    double mass         = ((heavyQuark.idAbs() == 4) ? mc : mb);
    bool is1A           = (heavyQuark.pz()>0.0);
    // Find the colour partner
    for (int j=0; j<(int)parts.size(); j++) if (j!=isToCheck[i]) {
        if ( (parts[j].col() != hQcol) && (parts[j].acol() != hQcol) ) continue;
        Particle colPartner = parts[i];
        double sHqCp        = m2(heavyQuark, colPartner);
        double Q2max        = 0.0;
        if (colPartner.isFinal()) 
          Q2max = trialIFSplitA.getQ2max(sHqCp, colPartner.e(), is1A ? eAused : eBused);
        else 
          Q2max = trialIISplitA.getQ2max(sHqCp, colPartner.e(), is1A ? eAused : eBused);
        // Phase space limit is below the mass
        if (sqrt(Q2max) < mass) return false;
        if (colPartner.isFinal()) { // Check for energy exceeding beam energy
          double eA       = heavyQuark.e();
          double eamax    = ( is1A ? (0.98*eBeamA - (eAused-eA)) : 
            (0.98*eBeamB - (eBused-eA)) ); // Allowed maxima
          double sjkmax   = sHqCp*(eamax-eA)/eA;
          double sakmax   = (sHqCp+sjkmax-(mass*mass)); // sajmin = mass^2
          if ( (sjkmax < 0.5) || (sakmax < 0.5) ) return false; // Invariant > 0.5 GeV !
        }
      }
  }
  // All clear
  return true;
}
  
//********* 
    
// Method to check if we have heavy quark left after passing the evolution window
bool SpaceAntShower::heavyQuarkLeft(double qTrial) {
  // We don't want to get rid of the quarks anyway
  if (massCorrectionLevel == 0) return false;
  // We are above mb
  if (qTrial > 1.02*mb) return false;
  bool foundQuark = false;
  // Loop over systems
  for (int iSys = 0; iSys < (int)branchelementals.size(); iSys++) {
    // Loop over antennae
    for (int iAnt = 0; iAnt < (int)branchelementals[iSys].size(); ++iAnt) {
      BranchelementalISR* trialPtr = &branchelementals[iSys][iAnt];
      int id1     = trialPtr->old1.idAbs();
      int id2     = trialPtr->old2.idAbs();
      bool isIni2 = !trialPtr->old2.isFinal();
      bool foundQuarkNow = false;
      int splitGenTndex  = -1;
      if ( (id1 > nFlavZeroMass) && (id1 < 6) ) {
        double mass = ((id1 == 4) ? mc : mb);
        // Safety of 2%
        if (qTrial < (1.02*mass)) {
          foundQuarkNow = true;
          // Find the index of the trial generator for splitting
          for (int indx = 0; indx < (int)trialPtr->nTrialGenerators(); ++indx) {
            if ( (trialPtr->getPhysIndex(indx) == iQXsplitIF) || 
              (trialPtr->getPhysIndex(indx) == iQXsplitII) ) {
              splitGenTndex = indx;
              trialPtr->scaleSav[indx] = mass; // Set the scale
            }
          }
        }
      }
      if ( isIni2 && (id2 > nFlavZeroMass) && (id2 < 6) ) {
        double mass = ((id2 == 4) ? mc : mb);
        // Safety of 2%
        if (qTrial < (1.02*mass)) {
          foundQuarkNow = true;
          // Find the index of the trial generator for splitting
          for (int indx = 0; indx < (int)trialPtr->nTrialGenerators(); ++indx) {
            if (trialPtr->getPhysIndex(indx) == iQXsplitII) {
              splitGenTndex = indx;
              trialPtr->scaleSav[indx] = mass; // Set the scale
            }
          }
        }
      }
      if (foundQuarkNow && (splitGenTndex>=0)) {
        winnerPtr  = trialPtr;
        iSysWin    = iSys;
        indxWin    = splitGenTndex;
        foundQuark = foundQuarkNow;
      }
      else if (foundQuarkNow) {
        if (verbose >=1) {
          printOut("SpaceAntShower::heavyQuarkLeft",
            "Something went wrong! Found heavy quark but no splitting trial"
            "generator! Not going to force a splitting!");
          trialPtr->list();
          cout << "     Current scale = " << qTrial << endl;
        }
      }
    }
  }
  return foundQuark;
}

} // end namespace VINCIA
